/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: uart_data_received_handle.h
* @Description: uart data receive handle
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-13
* @LastEditTime: 2022-11-13
*/
#ifndef __UART_DATA_RECEIVED_HANDLE_H
#define __UART_DATA_RECEIVED_HANDLE_H

#include "tuya_cloud_types.h"

#ifdef __cplusplus
	extern "C" {
#endif

#define SERIAL_PROTOCOL_UART_ID            TUYA_UART_NUM_0

#define UART_HEAD_1                        0x55
#define UART_HEAD_2                        0xaa
#define SERIAL_PROTOCOL_HDR                (UART_HEAD_1<<8 | UART_HEAD_2)

#define MAX_ZIGBEE_PAYLOAD_SIZE            64      ///<zigbee frame max payload size
#define MAX_MANU_NAME_SIZE                 34      ///<manufacture name max payload size
#define DEFAULT_PID_LEN                    8       ///<product id default length

#define SERIAL_PROTOCOL_VER                0x02
#define MAX_SERIAL_PAYLOAD_SIZE            128
#define PROTOCOL_EXT_SIZE                  0x09

#define HEAD_FIRST                         0
#define HEAD_SECOND                        1
#define PROTOCOL_VERSION                   2
#define SEQ_LOCATION_1                     3
#define SEQ_LOCATION_2                     4
#define CMD_LOCATION                       5
#define LENGTH_HIGH                        6
#define LENGTH_LOW                         7
#define DATA_START                         8

#define SERIAL_RF_SEND_NUM                 100      ///<RF test send number
#define SCAN_RSSI_PERIOD                   50      ///<scan rssi period 20ms
#define SCAN_RSSI_TIMEOUT                  SCAN_RSSI_PERIOD*SERIAL_RF_SEND_NUM*2 ///<scan rssi timeout 2000ms

//serial cmd id
#define MCU_RESET_TO_FACTORY_CMD           0x00
#define PRODUCT_INFO_CMD                   0x01
#define ZIGBEE_STATE_CMD                   0x02
#define ZIGBEE_CFG_CMD                     0x03
#define ZIGBEE_DATA_REQ_CMD                0x04
#define ACK_GATEWAY_SEND_CMD               0x05
#define DATA_REPORT_CMD                    0x06
#define FUNC_TEST_CMD                      0x08
#define SCENE_SWITCH_NUM_GET_CMD           0x09
#define SCENE_SWITCH_ID_REPORT_CMD         0x0A
#define MCU_OTA_VER_REQ_CMD                0x0B
#define MCU_OTA_NOTIFY_CMD                 0x0C
#define MCU_OTA_DATA_REQ_CMD               0x0D
#define MCU_OTA_END_CMD                    0x0E
#define CHECK_ZIGBEE_NETWORK_CMD           0x20
#define TIME_GET_CMD                       0x24
#define CHECK_GW_NETWORKING_ST_CMD         0x25
#define SET_ZIGBEE_NETWORK_INFO_CMD        0x26
#define MCU_DATA_BROADCAST_SEND_CMD        0x27
#define READ_MCU_DP_DATA_CMD               0x28
#define BEACON_TEST_NOTIFY_CMD             0x29
#define ZIGBEE_DATA_REQ_GROUP_CMD          0x2a
#define SET_FROM_WAKE_UP_TO_SEND_TIME_CMD  0x2b

#define MULTICAST_SET_CMD                  0x41
#define STANDARD_MULTICAST_CMD             0x42
#define PRIVATE_MULTICAST_CMD              0x43

#define MAX_MCU_POLL_TIMEOUT               60000
#define GET_GW_NETWORK_ST_TIMEOUT          3000

typedef enum
{
    NO_NWK_JOINING_E_CODE = 0x65,
    IN_NWK_JOINING_E_CODE,
    UART_FRAME_ERROE_E_CODE,
    UART_ACK_TIMMEOUT_E_CODE,
    GATEWAY_RSP_E_CODE,
    MCU_LONG_WAKEUP_E_CODE
}USER_ERROR_CODE_E;

typedef enum
{
    MF_TEST_MODULE,
    NWK_STATE_OK,
    NWK_STATE_ERROR
}WORK_STATE_E;

typedef enum
{
    NWK_NOT_JOIN = 0,
    NWK_JOINED,
    NWK_INFO_ERROR,
    NWK_JOINING
}NWK_STATE_E;

typedef enum {
    UART_ST_HEAD_1,
    UART_ST_HEAD_2,
	UART_ST_VER,
	UART_ST_SEQ1,
	UART_ST_SEQ2,
	UART_ST_CMD,
    UART_ST_DATA_LEN_1,
    UART_ST_DATA_LEN_2,
    UART_ST_DATA,
    UART_ST_CRC,
}UART_ST_T;


typedef struct {
    UINT16_T cmd_hdr;                        ///<serial protocol command header
    UINT8_T  version;                        ///<serial protocol version
    UINT16_T seq_num;                        ///<frame sequence number
    UINT8_T cmd;                              ///<serial protocol command type
    UINT16_T  data_len;                       ///<serial protocol data length
    UINT8_T  data[MAX_SERIAL_PAYLOAD_SIZE];  ///<serial protocol data
    UINT8_T  crc;                            ///<serial protocol data frame crc
}uart_protocol_cmd_t;

VOID_T uart_rx_callback(TUYA_UART_NUM_E port_id, VOID_T *buff, UINT16_T len);

/**
* @description: get get time seq
* @param {type} none
* @return: seq get time seq
*/
UINT16_T get_time_seq_get(VOID_T);

/**
* @description: set get time seq
* @param {seq} get time seq
* @return: none
*/
VOID_T get_time_seq_set(UINT16_T seq);

/**
* @description: get current device work state
* @param {type} none
* @return: WORK_STATE_E current work state
*/
WORK_STATE_E device_work_state_get(VOID_T);

/**
* @description: set device work state
* @param {state} WORK_STATE_E state
* @return: none
*/
VOID_T device_work_state_set(WORK_STATE_E state);

/**
* @description: get network status seq
* @param {type} none
* @return: seq network status seq
*/
UINT16_T gw_network_st_seq_get(VOID_T);

/**
* @description: set network status seq
* @param {seq} network status seq
* @return: none
*/
VOID_T gw_network_st_seq_set(UINT16_T seq);

/**
* @description: get mcu version
* @param {type} none
* @return: mcu version
*/
UINT8_T mcu_version_get(VOID_T);

/**
* @description: get user application data send sequence numbers
* @param {type} none
* @return: sequence number
*/
UINT16_T app_seq_num_get(VOID_T);

/**
* @description: network state get
* @param none
* @return: network state
*/
NWK_STATE_E user_network_state_get(VOID_T);

/**
* @description: get rf test sequence numbers
* @param {type} none
* @return: sequence number
*/
UINT16_T rf_test_seq_get(VOID_T);

/**
* @description: send network state to the MCU when the zigbee state is changed;
* @param {nwk_status} network status enum type, 0x00: not join, 0x01: joined;
* @return: none
*/
VOID_T report_zigbee_network_state_to_mcu(NWK_STATE_E nwk_state, BOOL_T send);

/**
* @description: delay report mcu version
* @param {delay_ms} delay time in ms
* @return: none
*/
VOID_T delay_report_mcu_version(UINT32_T delay_ms);

/**
* @description: stop gateway status check timeout event
* @param none
* @return: none
*/
VOID_T stop_gw_st_check_timeout_event(VOID_T);

#ifndef TUYA_ZG_ROUTER
/**
* @description: get wake up time
* @param {type} none
* @return: wake up time
*/
UINT16_T wake_up_time_get(VOID_T);

/**
* @description: set wake up time
* @param {time} wake up time
* @return: none
*/
VOID_T wake_up_time_set(UINT16_T time);
#endif

/**
* @description: serial protocol send data confirm queue init
* @param {type} none
* @return: none
*/
VOID_T uart_protocol_handle_init(VOID_T);

#ifdef __cplusplus
}
#endif

#endif

