/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: air_data_send.c
* @Description: air data send
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-13
* @LastEditTime: 2022-11-13
*/
#include "tal_memory.h"
#include "tal_sw_timer.h"

#include "air_data_send.h"
#include "air_data_received_handle.h"

#include "uart_data_received_handle.h"

#define AIR_DATA_SEND_DBG 0

#if AIR_DATA_SEND_DBG
#define air_data_send_debug(...)      tal_main_debug(__VA_ARGS__)
#else
#define air_data_send_debug(...)
#endif

#define TAL_SEND_DATA_TIMEOUT      1000
#define MAX_RESEND_TIMES           3
#define MCU_OTA_REQUEST_BUFFRE_LEN 30

//air send event
TIMER_ID mcu_ota_data_request_resend_event_id;

STATIC VOID_T mcu_ota_data_request_resend_event_handler(TIMER_ID timer_id, VOID_T *arg);

STATIC UINT8_T mcu_ota_request_backup[MCU_OTA_REQUEST_BUFFRE_LEN];
STATIC UINT8_T resend_times = 0;

STATIC VOID_T air_send_event_init(VOID_T);
STATIC VOID_T insert_private_command_send_buffer(UINT8_T cmd_id, TAL_ZG_SEND_DATA_T *send_data);
STATIC VOID_T mcu_ota_request_ack_receive_callback(TAL_SEND_ST_E st, TAL_ZG_SEND_DATA_T *data);

/**
 * @brief air send event init
 *
 * @param[in] none
 * @return none
 */
STATIC VOID_T air_send_event_init(VOID_T)
{
    tal_sw_timer_create(mcu_ota_data_request_resend_event_handler, NULL, &mcu_ota_data_request_resend_event_id);
}

/**
 * @brief mcu ota data request resend event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T mcu_ota_data_request_resend_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    if (resend_times < MAX_RESEND_TIMES) {
        air_data_send_debug("resend mcu ota data request, times %d\r\n", resend_times);
        air_mcu_ota_data_request_send(mcu_ota_request_backup[0], &mcu_ota_request_backup[2], mcu_ota_request_backup[1], TRUE);
    }

    resend_times++;
}

/**
 * @brief insert cluster command send buffer
 *
 * @param[out] send_data: send data, please refer to TAL_ZG_SEND_DATA_T
 * @param[in] cmd_id: cluster command id
 * @return None
 */
STATIC VOID_T insert_private_command_send_buffer(UINT8_T cmd_id, TAL_ZG_SEND_DATA_T *send_data)
{
    send_data->delay_time = 0;
    send_data->random_time = 0;

    send_data->manu_code = 0x0000;
    send_data->manu_spec = 0;

    send_data->command_id = cmd_id ;
    send_data->frame_type = ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER;
    send_data->direction = ZG_ZCL_DATA_SERVER_TO_CLIENT;
    send_data->qos = QOS_1;

    send_data->addr.mode = SEND_MODE_DEV;
    send_data->addr.type.dev.src_ep = 0x01;
    send_data->addr.type.dev.dst_ep = 0x01;
    send_data->addr.type.dev.cluster_id = CLUSTER_PRIVATE_TUYA_CLUSTER_ID;
    send_data->addr.type.dev.dst_addr = 0x0000;
}

/**
* @description: mcu ota request ack receive callback
* @param {st} send state
* @param {data} send data
* @return: none
*/
STATIC VOID_T mcu_ota_request_ack_receive_callback(TAL_SEND_ST_E st, TAL_ZG_SEND_DATA_T *data)
{
    if (SEND_ST_OK != st) {
        air_data_send_debug("receive mcu ota response status error\r\n");
        if (resend_times >= MAX_RESEND_TIMES) {
            air_data_send_debug("reach max resend time, return\r\n");
            resend_times = 0;
            return;
        }
        tal_sw_timer_start(mcu_ota_data_request_resend_event_id, 100, TAL_TIMER_ONCE);
    }else {
        if ((ZCL_ID_MCU_OTA_END == data->zcl_id)&&(SERIAL_PROTOCOL_MCU_OTA_END == data->command_id)) {
            air_data_send_debug("mcu ota end success\r\n");
            tal_sw_timer_stop(mcu_ota_data_request_resend_event_id);
            resend_times = 0;
            tal_system_memset(&mcu_ota_request_backup[0], 0, MCU_OTA_REQUEST_BUFFRE_LEN);
            return;
        }
        air_data_send_debug("receive mcu ota response, wait next ota request or end\r\n");
        tal_sw_timer_start(mcu_ota_data_request_resend_event_id, 3000, TAL_TIMER_ONCE);
    }
}

/**
* @description: air data send mcu DP read response to gateway
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @param {seq} seq
* @param {force_broad} force broadcast send message or not
* @return: none
*/
VOID_T air_mcu_DP_response_cmd_send(UINT8_T* data, 
                                               UINT8_T data_len, 
                                               UINT16_T seq, 
                                               BOOL_T force_broad)
{
    TAL_ZG_SEND_DATA_T send_data;

    if ((NULL == data) || (data_len > MAX_ZIGBEE_PAYLOAD_SIZE)){
      return;
    }

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    insert_private_command_send_buffer(SERIAL_PROTOCOL_DATA_RSP, &send_data);

    send_data.zcl_id = (UINT8_T)ZCL_ID_DP_RESPONSE;
    if (force_broad) {
        send_data.addr.type.dev.dst_addr = 0xffff;
        send_data.qos = QOS_0;
    } else {
        if(SEND_TYPE_UNICAST == air_message_type_get()){
            send_data.qos = QOS_VIP_0;
        }
        else{
            send_data.qos = QOS_0;
            send_data.random_time = 5000;
        }
    }

    send_data.data.private.data[0] = (UINT8_T)seq>>8;
    send_data.data.private.data[1] = (UINT8_T)seq;

    tal_system_memcpy(&(send_data.data.private.data[2]), data, data_len);
    send_data.data.private.len = data_len+2;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}

/**
* @description: air data send mcu data report to gateway
* @param {data} receive data ptr
* @param {data_len} receive data length
* @param {result} tal send result
* @return: none
*/
VOID_T air_mcu_DP_report_cmd_send(UINT8_T* data, 
                                            UINT8_T data_len, 
                                            TAL_SEND_RESULT_CB result)
{
    TAL_ZG_SEND_DATA_T send_data;

    if ((NULL == data) || (data_len > MAX_ZIGBEE_PAYLOAD_SIZE)){
      return;
    }

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    insert_private_command_send_buffer(SERIAL_PROTOCOL_DATA_REPORT, &send_data);

    send_data.zcl_id = (UINT8_T)ZCL_ID_DP_REPORT;
    if(SEND_TYPE_UNICAST != air_message_type_get()){
        send_data.qos = QOS_0;
        send_data.random_time = 500;
        send_data.addr.type.dev.dst_addr = 0xffff;
    }

    tal_system_memcpy(send_data.data.private.data, data, data_len);
    send_data.data.private.len = data_len;

    if (NULL != result) {
        tal_zg_send_data(&send_data, result, TAL_SEND_DATA_TIMEOUT);
    } else {
        tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
    }
}

/**
* @description: air send mcu version report to gateway
* @param {none}
* @return: none
*/
VOID_T air_mcu_version_report_send(VOID_T)
{
    TAL_ZG_SEND_DATA_T send_data;
    UINT16_T seq_num = app_seq_num_get();

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    insert_private_command_send_buffer(SERIAL_PROTOCOL_MCU_OTA_VER_RSP, &send_data);

    send_data.zcl_id = (UINT8_T)ZCL_ID_MCU_VERSION_REPORT;

    send_data.data.private.data[0] = seq_num >> 8;
    send_data.data.private.data[1] = seq_num;
    send_data.data.private.data[2] = mcu_version_get();
    send_data.data.private.len = 3;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}

/**
* @description: serial data response send callback, receive data from mcu and send to gateway
* @param {cmd_id} send data command id
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @param {re_send} resend flag
* @return: none
*/
VOID_T air_mcu_ota_data_request_send(UINT8_T cmd_id, 
                                                 UINT8_T* data, 
                                                 UINT8_T data_len,
                                                 BOOL_T re_send)
{
    TAL_ZG_SEND_DATA_T send_data;

    if ((NULL == data) || (data_len > MAX_ZIGBEE_PAYLOAD_SIZE)){
        return;
    }

    if (!re_send) {
        tal_sw_timer_stop(mcu_ota_data_request_resend_event_id);
        resend_times = 0;
        tal_system_memset(&mcu_ota_request_backup[0], 0, MCU_OTA_REQUEST_BUFFRE_LEN);
        mcu_ota_request_backup[0] = cmd_id;
        mcu_ota_request_backup[1] = data_len;
        tal_system_memcpy(&mcu_ota_request_backup[2], data, data_len);
    }

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    insert_private_command_send_buffer(cmd_id, &send_data);

    if (SERIAL_PROTOCOL_MCU_OTA_DATA_REQ == cmd_id) {
        air_data_send_debug("mcu ota data request, resend %d\r\n", re_send);
        send_data.zcl_id = (UINT8_T)ZCL_ID_MCU_OTA_DATA_REQUEST;
    } else {
        air_data_send_debug("mcu ota data end, resend %d\r\n", re_send);
        send_data.zcl_id = (UINT8_T)ZCL_ID_MCU_OTA_END;
    }

    tal_system_memcpy(send_data.data.private.data, data, data_len);
    send_data.data.private.len = data_len;

    tal_zg_send_data(&send_data, mcu_ota_request_ack_receive_callback, TAL_SEND_DATA_TIMEOUT);
}

/**
* @description: air data send time request to gateway
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @return: none
*/
VOID_T air_time_request_cmd_send(UINT8_T* data, UINT8_T data_len)
{
    TAL_ZG_SEND_DATA_T send_data;

    if ((NULL == data) || (data_len > MAX_ZIGBEE_PAYLOAD_SIZE)){
      return;
    }

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    insert_private_command_send_buffer(SERIAL_PROTOCOL_TIME_REQ, &send_data);

    send_data.zcl_id = (UINT8_T)ZCL_ID_TTME_REQUEST;

    tal_system_memcpy(send_data.data.private.data, data, data_len);
    send_data.data.private.len = data_len;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}

/**
* @description: air gateway network status request to gateway
* @param {* data} receive data ptr
* @param {data_len} receive data length
* @return: none
*/
VOID_T air_gateway_network_status_request_send(UINT8_T *data, UINT8_T data_len)
{
    TAL_ZG_SEND_DATA_T send_data;

    if ((NULL == data) || (data_len > MAX_ZIGBEE_PAYLOAD_SIZE)){
      return;
    }

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    insert_private_command_send_buffer(SERIAL_PROTOCOL_GW_NETWORK_ST_REQ, &send_data);

    send_data.zcl_id = (UINT8_T)ZCL_ID_GATEWAY_NETWORK_STATUS_REQUEST;

    tal_system_memcpy(send_data.data.private.data, data, data_len);
    send_data.data.private.len = data_len;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}

#if COMMON_LINK_SCENE
/**
* @description: air scene add response send
* @param {seq} request seq
* @return: none
*/
VOID_T air_scene_add_response_send(UINT16_T seq)
{
    TAL_ZG_SEND_DATA_T send_data;

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    insert_private_command_send_buffer(SERIAL_PROTOCOL_DATA_RSP, &send_data);

    send_data.zcl_id = (UINT8_T)ZCL_ID_SCENE_ADD_RESPONSE;

    send_data.data.private.data[0] = (UINT8_T)seq>>8;
    send_data.data.private.data[1] = (UINT8_T)seq;
    
    send_data.data.private.data[2] = 17;
    send_data.data.private.data[3] = 0;
    send_data.data.private.data[4] = 0;
    send_data.data.private.data[5] = 1;
    send_data.data.private.data[6] = 0;
    send_data.data.private.len = 8;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}

/**
* @description: air standard recall scene send
* @param {group_id} group id
* @param {scene_id} scene id
* @return: none
*/
VOID_T air_standard_recall_scene_send(UINT8_T group_id, UINT8_T scene_id)
{
    TAL_ZG_SEND_DATA_T send_data;

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));

    send_data.delay_time = 0;
    send_data.random_time = 0;

    send_data.manu_code = 0x0000;
    send_data.manu_spec = 0;

    send_data.command_id = CMD_RECALL_SCENE_COMMAND_ID;
    send_data.frame_type = ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER;
    send_data.direction = ZG_ZCL_DATA_CLIENT_TO_SERVER;
    send_data.qos = QOS_0;

    send_data.addr.mode = SEND_MODE_DEV;
    send_data.addr.type.dev.src_ep = 0x01;
    send_data.addr.type.dev.dst_ep = 0xff; //destination all ep
    send_data.addr.type.dev.cluster_id = CLUSTER_SCENES_CLUSTER_ID;
    send_data.addr.type.dev.dst_addr = 0xffff;

    send_data.zcl_id = (UINT8_T)ZCL_ID_GATEWAY_NETWORK_STATUS_REQUEST;

    send_data.data.private.data[1] = 0;
    send_data.data.private.data[0] = group_id;
    send_data.data.private.data[2] = scene_id;
    send_data.data.private.len = 3;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}

/**
* @description: air group broadcst standard command send
* @param {groupid} group id
* @param {cluster} cluster id
* @param {cmd} command id
* @param {payload} payload data
* @param {payload_len} payload length
* @return: none
*/
VOID_T air_group_broadcast_standard_cmd_send(UINT16_T groupid, UINT16_T cluster, UINT8_T cmd, UINT8_T * payload, UINT8_T payload_len)
{
    TAL_ZG_SEND_DATA_T send_data;

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));

    send_data.delay_time = 0;
    send_data.random_time = 0;

    send_data.manu_code = 0x0000;
    send_data.manu_spec = 0;

    send_data.command_id = cmd;
    send_data.frame_type = ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER;
    send_data.direction = ZG_ZCL_DATA_CLIENT_TO_SERVER;
    send_data.qos = QOS_1;

    send_data.addr.mode = SEND_MODE_GROUP;
    send_data.addr.type.group.src_ep = 0x01;
    send_data.addr.type.group.dst_ep = 0xff; //destination all ep
    send_data.addr.type.group.cluster_id = cluster;
    send_data.addr.type.group.group_id = groupid;

    send_data.zcl_id = (UINT8_T)ZCL_ID_GROUP_BROADCAST_STANDARD;

    tal_system_memcpy(send_data.data.private.data, payload, payload_len);
    send_data.data.private.len = payload_len;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}

/**
* @description: air group broadcst private command send
* @param {groupid} group id
* @param {payload} payload data
* @param {payload_len} payload length
* @return: none
*/
VOID_T air_group_broadcast_private_cmd_send(UINT16_T groupid, UINT8_T * payload, UINT8_T payload_len)
{
    TAL_ZG_SEND_DATA_T send_data;
    UINT16_T seqnum = app_seq_num_get();

    tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));

    send_data.delay_time = 0;
    send_data.random_time = 0;

    send_data.manu_code = 0x0000;
    send_data.manu_spec = 0;

    send_data.command_id = 0x00;
    send_data.frame_type = ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER;
    send_data.direction = ZG_ZCL_DATA_CLIENT_TO_SERVER;
    send_data.qos = QOS_VIP_1;

    send_data.addr.mode = SEND_MODE_GROUP;
    send_data.addr.type.group.src_ep = 0x01;
    send_data.addr.type.group.dst_ep = 0xff; //destination all ep
    send_data.addr.type.group.cluster_id = CLUSTER_PRIVATE_TUYA_CLUSTER_ID;
    send_data.addr.type.group.group_id = groupid;

    send_data.zcl_id = (UINT8_T)ZCL_ID_GROUP_BROADCAST_PRIVATE;

    send_data.data.private.data[0] = (UINT8_T)((seqnum >> 8) & 0xFF);
    send_data.data.private.data[1] = (UINT8_T)(seqnum  & 0xFF);

    tal_system_memcpy(&send_data.data.private.data[2], payload, payload_len);
    send_data.data.private.len = payload_len + 2;

    tal_zg_send_data(&send_data, NULL, TAL_SEND_DATA_TIMEOUT);
}
#endif

/**
 * @brief air send init
 *
 * @param[in] none
 * @return none
 */
VOID_T air_send_init(VOID_T)
{
    tal_system_memset(&mcu_ota_request_backup[0], 0, MCU_OTA_REQUEST_BUFFRE_LEN);
    air_send_event_init();
}

