/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: app_user_flash.c
* @Description: app user flash
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-30
* @LastEditTime: 2022-11-30
*/
#include "tal_nv_flash.h"
#include "tal_memory.h"

#include "app_user_flash.h"

//default active byte
#define DEFAULT_ACTIVE                           0xAA
//flash active id
#define USER_FLASH_ACTIVE_ID                     1
//uart baudrate
#define USER_BAUDRATE_ID                         2
//modle network info
#define USER_MODLE_NETWORK_INFO_ID               3
//rf power
#define USER_RF_POWER_ID                         4
#ifndef TUYA_ZG_ROUTER
//user flash netowrk type id
#define USER_WAKE_UP_TIME_ID                     5
#endif

#if COMMON_LINK_SCENE
#define USER_SCENE_DATA_ID                       6
#define USER_NETWORK_INFO_ID                     7
#endif

STATIC UINT8_T app_user_flash_valid_read(VOID_T);
STATIC BOOL_T app_user_flash_valid_write(UINT8_T active);

/**
 * @description: read flash valid status
 * @param[in] none
 * @return: active byte
 */
STATIC UINT8_T app_user_flash_valid_read(VOID_T)
{
    UINT8_T active = 0;
    tal_kv_flash_read(USER_FLASH_ACTIVE_ID, 1, &active);
    return active;
}

/**
 * @description: write user flash valid
 * @param {active} active byte
 * @return: TRUE when write success or FALSE
 */
STATIC BOOL_T app_user_flash_valid_write(UINT8_T active)
{
    TAL_KV_STATUS_T status = TAL_KV_SUCC;
    status = tal_kv_flash_write(USER_FLASH_ACTIVE_ID, 1, &active);
    if (TAL_KV_SUCC == status) {
        return TRUE;
    }
    return FALSE;
}

/**
 * @description: check if user flash valid or not
 * @param[in] none
 * @return: TRUE if user flash in valid or FALSE
*/
BOOL_T is_app_user_flash_valid(VOID_T)
{
    UINT8_T active = app_user_flash_valid_read();

    if (active != DEFAULT_ACTIVE) {
        return FALSE;
    }
    return TRUE;
}

/**
 * @description: read serial uart baudrate
 * @param[in] none
 * @return: baudrate
 */
UINT32_T app_user_flash_baudrate_read(VOID_T)
{
    UINT8_T temp[4] = {0, 0, 0, 0};
    UINT32_T baudrate = 0;
    tal_kv_flash_read(USER_BAUDRATE_ID, 4, &temp[0]);
    baudrate = (temp[3]<<24)|(temp[2]<<16)|(temp[1]<<8)|temp[0];

    return baudrate;
}

/**
 * @description: write serial uart baudrate
 * @param {rate} baudrate
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_baudrate_write(UINT32_T rate)
{
    UINT8_T temp[4] = {0, 0, 0, 0};
    TAL_KV_STATUS_T status = TAL_KV_SUCC;
    if (!is_app_user_flash_valid()) {
        app_user_flash_valid_write(DEFAULT_ACTIVE);
    }
    temp[0] = (UINT8_T)rate;
    temp[1] = (UINT8_T)(rate>>8);
    temp[2] = (UINT8_T)(rate>>16);
    temp[3] = (UINT8_T)(rate>>24);
    status = tal_kv_flash_write(USER_BAUDRATE_ID, 4, &temp[0]);
    if (TAL_KV_SUCC != status) {
        return FALSE;
    }
    return TRUE;
}

/**
 * @description: read serial module config info
 * @param[in] none
 * @return: read success or not
 */
BOOL_T app_user_flash_module_config_info_read(MODULE_CONFIG_INFO_T *info)
{
    UINT8_T temp[20];
    TAL_KV_STATUS_T status = TAL_KV_SUCC;

    status = tal_kv_flash_read(USER_MODLE_NETWORK_INFO_ID, SIZEOF(MODULE_CONFIG_INFO_T), &temp[0]);

    if (TAL_KV_SUCC != status) {
        return FALSE;
    }
    tal_system_memcpy(info, &temp[0], SIZEOF(MODULE_CONFIG_INFO_T));

    return TRUE;
}

/**
 * @description: write module config info
 * @param {rate} baudrate
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_module_config_info_write(MODULE_CONFIG_INFO_T info)
{
    UINT8_T temp[20];
    TAL_KV_STATUS_T status = TAL_KV_SUCC;
    if (!is_app_user_flash_valid()) {
        app_user_flash_valid_write(DEFAULT_ACTIVE);
    }
    tal_system_memset(&temp[0], 0, 20);
    tal_system_memcpy(&temp[0], &info, SIZEOF(MODULE_CONFIG_INFO_T));

    status = tal_kv_flash_write(USER_MODLE_NETWORK_INFO_ID, SIZEOF(MODULE_CONFIG_INFO_T), &temp[0]);
    if (TAL_KV_SUCC != status) {
        return FALSE;
    }
    return TRUE;
}

/**
 * @description: read RF power
 * @param[in] none
 * @return: active byte
 */
UINT8_T app_user_flash_rf_power_read(VOID_T)
{
    UINT8_T power = 0;
    tal_kv_flash_read(USER_RF_POWER_ID, 1, &power);
    return power;
}

/**
 * @description: write user flash active
 * @param {active} active byte
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_rf_power_write(UINT8_T power)
{
    TAL_KV_STATUS_T status = TAL_KV_SUCC;
    status = tal_kv_flash_write(USER_RF_POWER_ID, 1, &power);
    if (TAL_KV_SUCC == status) {
        return TRUE;
    }
    return FALSE;
}

#ifndef TUYA_ZG_ROUTER
/**
 * @description: read wakeup time
 * @param[in] none
 * @return: time
 */
UINT16_T app_user_flash_wakeup_time_read(VOID_T)
{
    UINT8_T temp[2] = {0, 0};
    UINT16_T wake_time = 0;
    tal_kv_flash_read(USER_WAKE_UP_TIME_ID, 2, &temp[0]);
    wake_time = (temp[1]<<8)|temp[0];

    return wake_time;
}

/**
 * @description: write wake up time
 * @param {time} wakeup time
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_wakeup_time_write(UINT16_T time)
{
    UINT8_T temp[2] = {0, 0};
    TAL_KV_STATUS_T status = TAL_KV_SUCC;
    if (!is_app_user_flash_valid()) {
        app_user_flash_valid_write(DEFAULT_ACTIVE);
    }
    temp[0] = (UINT8_T)time;
    temp[1] = (UINT8_T)(time>>8);
    status = tal_kv_flash_write(USER_WAKE_UP_TIME_ID, 2, &temp[0]);
    if (TAL_KV_SUCC != status) {
        return FALSE;
    }
    return TRUE;
}
#endif

#if COMMON_LINK_SCENE
/**
 * @description: read scene data
 * @param[in] none
 * @return: read success or not
 */
BOOL_T app_user_scene_data_read(UINT8_T *data)
{
    TAL_KV_STATUS_T status = TAL_KV_SUCC;

    status = tal_kv_flash_read(USER_SCENE_DATA_ID, (MAX_USER_SCENE_ITEM*4), data);

    if (TAL_KV_SUCC != status) {
        return FALSE;
    }

    return TRUE;
}

/**
 * @description: scene data write
 * @param {data} scene data
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_scene_data_write(UINT8_T *data)
{
    TAL_KV_STATUS_T status = TAL_KV_SUCC;
    if (!is_app_user_flash_valid()) {
        app_user_flash_valid_write(DEFAULT_ACTIVE);
    }

    status = tal_kv_flash_write(USER_SCENE_DATA_ID, (MAX_USER_SCENE_ITEM*4), data);
    if (TAL_KV_SUCC != status) {
        return FALSE;
    }
    return TRUE;
}

/**
 * @description: read network info
 * @param {data} scene data
 * @return: read success or not
 */
BOOL_T app_user_flash_network_info_read(UINT8_T *data)
{
    TAL_KV_STATUS_T status = TAL_KV_SUCC;

    status = tal_kv_flash_read(USER_NETWORK_INFO_ID, SIZEOF(USER_NWK_INFO_T), data);

    if (TAL_KV_SUCC != status) {
        return FALSE;
    }

    return TRUE;
}

/**
 * @description: network info data write
 * @param {data} scene data
 * @return: TRUE when write success or FALSE
 */
BOOL_T app_user_flash_network_info_write(UINT8_T *data)
{
    TAL_KV_STATUS_T status = TAL_KV_SUCC;
    if (!is_app_user_flash_valid()) {
        app_user_flash_valid_write(DEFAULT_ACTIVE);
    }

    status = tal_kv_flash_write(USER_NETWORK_INFO_ID, SIZEOF(USER_NWK_INFO_T), data);
    if (TAL_KV_SUCC != status) {
        return FALSE;
    }
    return TRUE;
}

#endif

