#include "tkl_network_manager.h"
#include "tkl_network_recovery.h"

#include "tal_memory.h"
#include "tal_sw_timer.h"
#include "tal_zcl_group.h"
#include "tal_network_mgr.h"
#include "tal_data_send.h"
#include "tal_data_receive.h"
#include "tal_attribute_rw.h"
#include "tal_endpoint_register.h"
#include "tal_reset_factory.h"
#include "tal_heartbeat.h"
#include "tal_firmware_cfg.h"
#include "tal_uart.h"
#include "tal_zll_commissioning.h"
#include "tal_error_code.h"
#include "tal_mf_test.h"
#include "tal_zcl_scene.h"
#include "tuya_mem_heap.h"

#include "app_common.h"
#include "app_dev_register.h"
#include "app_user_flash.h"

#include "uart_data_send.h"
#include "uart_data_received_handle.h"

#include "air_data_send.h"
#include "air_data_received_handle.h"

#include "baudrate_adaptive.h"

#ifdef TUYA_ZG_ROUTER
#include "dp_data_backup.h"
#else
#include "app_sleep_device_config.h"
#endif

#define USER_DEBUG_UART TUYA_UART_NUM_1
#define USER_TOKEN_TEST               0

//serial common event id
TIMER_ID power_on_event_id;
TIMER_ID user_joined_event_id;
TIMER_ID user_time_sync_event_id;
TIMER_ID test_event_id;

//serial common event handler
STATIC VOID_T __power_on_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T user_joined_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T user_time_sync_event_handler(TIMER_ID timer_id, VOID_T *arg);
#if USER_TOKEN_TEST
STATIC VOID_T test_event_handler(TIMER_ID timer_id, VOID_T *arg);
#endif

#if MAIN_CALLBACK_DEBUG
STATIC VOID_T __tuya_uart_debug_init(VOID_T);
#endif

#if COMMON_LINK_SCENE
STATIC OPERATE_RET group_cluster_command_parse_handler(UINT8_T cmd, ZG_DELIVERY_MODE_E mode, UINT8_T *buff, UINT8_T buff_len);
#endif

STATIC VOID_T tuya_sdk_event_init(VOID_T);
VOID_T common_link_uart_init(UINT_T baudrate);
STATIC VOID_T tuya_rf_power_check_and_set(VOID_T);
STATIC VOID_T __network_info_print(VOID_T);

#define APP_HEAP_SIZE 10*1024
STATIC UINT8_T gs_heap[APP_HEAP_SIZE];
VOID_T tkl_memory_cfg(UINT8_T **out_buff, UINT32_T *out_buff_len)
{
    *out_buff = gs_heap;
    *out_buff_len = APP_HEAP_SIZE;
}

/**
 * @brief power on event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T __power_on_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    UINT32_T baudrate = 0;
    baudrate = app_user_flash_baudrate_read();

    if ((DEFAULT_SUPPORT_BAUDRATE_9600 != baudrate)&&(DEFAULT_SUPPORT_BAUDRATE_115200 != baudrate)) {
        //flash baudrate invalid, use default 115200 start baudrate check
        baudrate = DEFAULT_SUPPORT_BAUDRATE_115200;
    }
    common_link_uart_init(baudrate);
}

/**
 * @brief user joined event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T user_joined_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    TAL_ZG_NWK_STATUS_E  state = tal_zg_nwk_status_get();
    if ((TAL_ZG_NWK_POWER_ON_ONLINE == state) ||
            (TAL_ZG_NWK_JOIN_OK == state) ||
            (TAL_ZG_NWK_REJOIN_OK == state) ||
            (TAL_ZG_NWK_LOST == state)){
        report_zigbee_network_state_to_mcu(NWK_JOINED, FALSE);
    }
    tal_sw_timer_start(user_time_sync_event_id, 9500, TAL_TIMER_ONCE);
}

/**
 * @brief time sync event handler
 *
 * @param[in] timer_id: event timer id
 * @param[in] arg: event data
 * @return NULL
 */
STATIC VOID_T user_time_sync_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    UINT16_T seq_num = app_seq_num_get();
    UINT8_T data[2] = {0};

    data[0] = (seq_num >> 8) & 0xff;
    data[1] = (seq_num & 0xff);
    air_time_request_cmd_send(data, 2);
}

#if USER_TOKEN_TEST
STATIC VOID_T test_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    STATIC UINT8_T first_flag = 0;
    if (!first_flag) {
        first_flag = 1;
        UINT8_T temp_data[177];
        tal_nv_flash_write(1, 109, 177, &temp_data[0]);
    } else {
        UINT8_T temp_data[13] = {0, 0, 0};
        tal_system_memset(&temp_data[0], 0, 13);
        tal_nv_flash_write(1, 110, 13, &temp_data[0]);
        tal_sw_timer_stop(test_event_id);
    }
    UINT8_T test_data[4] = {0, 0, 0, 0};
    tal_flash_read(0x110e6000, &test_data[0], 4);
    tal_main_debug("data[%x, %x, %x, %x]\r\n", test_data[0], test_data[1], test_data[2], test_data[3]);
}
#endif

/**
 * @brief tuya SDK event init
 *
 * @param[in] NULL
 * @return NULL
 */
STATIC VOID_T tuya_sdk_event_init(VOID_T)
{
    tal_sw_timer_create(__power_on_event_handler, NULL, &power_on_event_id);
    tal_sw_timer_create(user_joined_event_handler, NULL, &user_joined_event_id);
    tal_sw_timer_create(user_time_sync_event_handler, NULL, &user_time_sync_event_id);
#if USER_TOKEN_TEST
    tal_sw_timer_create(test_event_handler, NULL, &test_event_id);
#endif
}

/**
 * @brief check and set rf power
 *
 * @param[in] NULL
 * @return NULL
 */
STATIC VOID_T tuya_rf_power_check_and_set(VOID_T)
{
    //set rf power
    UINT8_T rf_power = 0;
    rf_power = app_user_flash_rf_power_read();
    tal_main_debug("rf power read %d\r\n", rf_power);
    if ((rf_power >= MIN_RF_POWER)&&(rf_power <= MAX_RF_POWER)) {
        tkl_zg_radio_power_set(rf_power);
    } else {
        tkl_zg_radio_power_set(DEFAULT_RF_POWER);
    }
}

STATIC VOID_T __network_info_print(VOID_T)
{
    TKL_NWK_BASIC_INFO_T nwk_info;
    tkl_zg_nwk_base_info_get(&nwk_info);
    tal_main_debug("power_on_online, \npan-id=0x%04X \nch=%d \naddr=%04X \ntx_power=%d\r\n", 
                    nwk_info.panid, 
                    nwk_info.radio_channel,
                    nwk_info.nwk_addr,
                    nwk_info.radio_power);
}

VOID_T common_link_uart_init(UINT_T baudrate)
{
    TAL_UART_CFG_T uart_cfg;
    tal_system_memset(&uart_cfg, 0, SIZEOF(TAL_UART_CFG_T));
        uart_cfg.rx_buffer_size = 128,
        uart_cfg.open_mode = 0,
        uart_cfg.base_cfg.baudrate = baudrate,
        uart_cfg.base_cfg.parity = TUYA_UART_PARITY_TYPE_NONE,
        uart_cfg.base_cfg.databits = TUYA_UART_DATA_LEN_8BIT;
        uart_cfg.base_cfg.stopbits = TUYA_UART_STOP_LEN_1BIT,
        uart_cfg.base_cfg.flowctrl = TUYA_UART_FLOWCTRL_NONE;

        tal_uart_deinit(SERIAL_PROTOCOL_UART_ID);
        tal_uart_init(SERIAL_PROTOCOL_UART_ID, &uart_cfg);
        tal_uart_rx_reg_irq_cb(SERIAL_PROTOCOL_UART_ID, &uart_rx_callback);
}

#if MAIN_CALLBACK_DEBUG
/**
 * @brief 
 * 
 * @param str 
 * @return VOID_T 
 */
STATIC VOID_T app_uart_output(IN CONST CHAR_T *str)
{
    tal_uart_write((TUYA_UART_NUM_E)USER_DEBUG_UART, (CONST UINT8_T *)str, strlen(str));
}

STATIC VOID_T __tuya_uart_debug_init(VOID_T)
{
    TAL_UART_CFG_T uart_cfg;
    tal_system_memset(&uart_cfg, 0, SIZEOF(TAL_UART_CFG_T));
        uart_cfg.rx_buffer_size = 128,
        uart_cfg.open_mode = 0,
        uart_cfg.base_cfg.baudrate = 115200,
        uart_cfg.base_cfg.parity = TUYA_UART_PARITY_TYPE_NONE,
        uart_cfg.base_cfg.databits = TUYA_UART_DATA_LEN_8BIT;
        uart_cfg.base_cfg.stopbits = TUYA_UART_STOP_LEN_1BIT,
        uart_cfg.base_cfg.flowctrl = TUYA_UART_FLOWCTRL_NONE;

        tal_uart_init(USER_DEBUG_UART, &uart_cfg);
        tal_log_create_manage_and_init(TAL_LOG_LEVEL_DEBUG, 128, Tl_printf);
}
#endif

#if COMMON_LINK_SCENE
/**
 * Af_command_parse_handle
 * Cluster: serial protocol private cluster command parse callback
 * @param {cmd} air command of private cluster
 * @param {mode} delivery mode
 * @param {buff} air data 
 * @param {buff_len} air data len
 * @return operate status
 */
STATIC OPERATE_RET group_cluster_command_parse_handler(UINT8_T cmd, ZG_DELIVERY_MODE_E mode, UINT8_T *buff, UINT8_T buff_len)
{
    if (NULL == buff) {
        return OPRT_COM_ERROR;
    }

    switch (cmd){
        case 0xF0: {
            if (0 == (buff_len - 1)%3) {
                UINT8_T sum = buff[0];
                UINT16_T gid = 0;
                UINT8_T kid = 0;
                if(sum > 8 || (sum*3+1) != buff_len){
                    return OPRT_COM_ERROR;
                }
                for (UINT8_T i = 0; i<sum; i++) {
                    gid = (UINT16_T)buff[2 + i*3] << 8 | buff[1 + i*3];
                    kid = buff[3 + i*3];
                    uart_multicast_set_cmd_send(kid, gid, 0);
                }
            }
        }
        break;

        case CMD_REMOVE_GROUP_COMMAND_ID: {
            UINT16_T group_id = (UINT16_T)((buff[1] << 8) | buff[0]);
            app_user_scene_info_clear(group_id);
        }
        break;

        case CMD_REMOVE_ALL_GROUPS_COMMAND_ID: {
            app_user_scene_info_clear(0xffff);
        }
        break;

        default:
        break;
    }
    return OPRT_OK;
}

extern UINT16_T tkl_zg_nwk_panid_get(VOID_T);
extern UINT16_T tkl_zg_nwk_address_get(VOID_T);
extern UINT8_T tkl_zg_radio_channel_get(VOID_T);

STATIC VOID_T app_network_info_check(VOID_T)
{
    UINT8_T temp_data[SIZEOF(USER_NWK_INFO_T)];
    USER_NWK_INFO_T net_info;
    tal_system_memset(&temp_data[0], 0, SIZEOF(USER_NWK_INFO_T));

    app_user_flash_network_info_read(&temp_data[0]);
    tal_system_memcpy(&net_info, &temp_data[0], SIZEOF(USER_NWK_INFO_T));

    if ((0xAA == net_info.valid)&&(tkl_zg_nwk_panid_get() == net_info.panid)) {
        return;
    }

    if ((0xAA == net_info.valid)&&(tkl_zg_nwk_panid_get() != net_info.panid)) {
        app_user_scene_info_clear(0xffff);
    }
    net_info.valid = 0xAA;
    net_info.channel = tkl_zg_radio_channel_get();
    net_info.nwk_addr = tkl_zg_nwk_address_get();
    net_info.panid = tkl_zg_nwk_panid_get();
    tal_system_memcpy(&temp_data[0], &net_info, SIZEOF(USER_NWK_INFO_T));
    app_user_flash_network_info_write(&temp_data[0]);

}
#endif

OPERATE_RET tuya_init_first(VOID_T)
{
    //uart init
#if MAIN_CALLBACK_DEBUG
    __tuya_uart_debug_init();
#endif
    //zigbee endpont init
#ifndef TUYA_ZG_ROUTER
        tkl_zg_nwk_recovery_disable(TRUE);
        app_sleep_end_device_config_load();
#endif
    app_dev_zigbee_init();
	tal_main_debug("/*********first init*********/\r\n");
    return OPRT_OK;
}

OPERATE_RET tuya_init_second(VOID_T)
{
    uart_queue_init();
    uart_protocol_handle_init();
    //event init
    tuya_sdk_event_init();
    air_data_received_handle_event_init();
    baudrate_adaptive_event_init();
    tal_main_debug("/*********second init*********/\r\n");
    return OPRT_OK;
}

OPERATE_RET tuya_init_third(VOID_T)
{
    tal_main_debug("/*********third init*********/\r\n");
    return OPRT_OK;
}

OPERATE_RET tuya_init_last(VOID_T)
{
    tal_main_debug("/*********last init*********/\r\n");
    //debug uart init
#if MAIN_CALLBACK_DEBUG
    __tuya_uart_debug_init();
#endif
#ifdef TUYA_ZG_ROUTER
    //back up dp data init
    backup_dp_data_init();
#endif

    baudrate_adaptive_init();
    air_send_init();

    //heartbeat type set to app version
    tal_heartbeat_type_set(HEARTBEAT_APP_VERSION);
    //heartbeat period set to 150s
    tal_heartbeat_period_set(150*1000UL);
    //delay send check product info
    baudrate_adaptive_start(4000, BAUDRATE_ADAPTIVE_TYPE_CHECK_INFO);
    //set rf power
    tuya_rf_power_check_and_set();

    tal_sw_timer_start(power_on_event_id, 10, TAL_TIMER_ONCE);

#if USER_TOKEN_TEST
    tal_sw_timer_start(test_event_id, 30000, TAL_TIMER_CYCLE);
#endif

    return OPRT_OK;
}

OPERATE_RET tuya_main_loop(VOID_T)
{
    return OPRT_OK;
}

VOID_T vSytickInterruptHook( VOID_T )
{
}

VOID_T vApplicationTickHook( VOID_T )
{
}

TAL_WRITE_RET_E tal_zg_pre_write_attribute_callback(UINT8_T ep_id,
                                                    UINT16_T cluster_id,
                                                    TAL_ATTR_REC_T *attr_rec)
{
    //tal_main_debug("pre write callback: cluster 0x%02x\r\n", cluster_id);
    return WRITE_RET_SUCCESS;
}

/**
 * @brief post-write attribute callback
 *
 * @param[in] ep_id:      endpoint id
 * @param[in] cluster_id: cluster id
 * @param[in] attr_rec:   point to attribute record 
 * @return write status
 */
TAL_WRITE_RET_E tal_zg_post_write_attribute_callback(UINT8_T ep_id, 
                                                                UINT16_T cluster_id,
                                                                TAL_ATTR_REC_T *attr_rec)
{
     return WRITE_RET_SUCCESS;
}

TAL_MSG_RET_E tal_zcl_general_msg_recv_callback(TAL_ZCL_MSG_T *msg)
{
    tal_main_debug("app gen msg cb: cluster 0x%02x, cmd %d, len %d\r\n", msg->cluster, msg->command, msg->length);

    return ZCL_MSG_RET_SUCCESS;
}

TAL_MSG_RET_E tal_zcl_specific_msg_recv_callback(TAL_ZCL_MSG_T *msg)
{
    if (NULL == msg) {
        return ZCL_MSG_RET_INVALID_VALUE;
    }

    switch (msg->cluster)
    {
        #if COMMON_LINK_SCENE
            case CLUSTER_GROUPS_CLUSTER_ID: {
                group_cluster_command_parse_handler(msg->command, msg->mode, msg->payload, msg->length);
                break;
            }
        #endif
        case CLUSTER_PRIVATE_TUYA_CLUSTER_ID: {
            air_private_cluster_command_parse_handle(msg->command, msg->mode, msg->payload, msg->length);
        }
        default:
            break;
    }
    return ZCL_MSG_RET_SUCCESS;
}

VOID_T tal_zg_nwk_status_changed_callback(TAL_ZG_NWK_STATUS_E status)
{
    switch(status)
    {
        case TAL_ZG_NWK_POWER_ON_LEAVE: { 
            tal_main_debug("---app nwk_power_on_leave---\r\n");
            break;
        }
        case TAL_ZG_NWK_POWER_ON_ONLINE: {
            tal_main_debug("---app nwk_power_on_onlie---\r\n");
            __network_info_print();
            if(!tal_sw_timer_is_running(user_joined_event_id)){
                tal_sw_timer_start(user_joined_event_id, 3500, TAL_TIMER_ONCE);
            }
            delay_report_mcu_version(5000);
            break;
        }
        case TAL_ZG_NWK_JOIN_START: {
            tal_main_debug("---app nwk_join_start---\r\n");
            report_zigbee_network_state_to_mcu(NWK_JOINING, FALSE);
            break;
        }
        case TAL_ZG_NWK_JOIN_OK: {
            tal_main_debug("---app nwk_join_ok---\r\n");
            __network_info_print();
            #if COMMON_LINK_SCENE
                app_network_info_check();
            #endif
            if(!tal_sw_timer_is_running(user_joined_event_id)){
                tal_sw_timer_start(user_joined_event_id, 500, TAL_TIMER_ONCE);
            }
            delay_report_mcu_version(5000);
            tuya_rf_power_check_and_set();
            break;
        }
        case TAL_ZG_NWK_REJOIN_OK: {
            tal_main_debug("---app nwk_rejoin_ok---\r\n");
            break;
        }
        case TAL_ZG_NWK_JOIN_TIMEOUT: {
            tal_main_debug("---app nwk_join_timeout---\r\n");
            report_zigbee_network_state_to_mcu(NWK_NOT_JOIN, FALSE);
            break;
        }
        case TAL_ZG_NWK_LOST: {
            tal_main_debug("---app nwk_lost---\r\n");
            break;
        }
        case TAL_ZG_NWK_REMOTE_LEAVE: {
            tal_main_debug("---app nwk_remote_leave---\r\n");
            report_zigbee_network_state_to_mcu(NWK_NOT_JOIN, FALSE);
            break;
        }
        case TAL_ZG_NWK_LOCAL_LEAVE: {
            tal_main_debug("---app nwk_local_leave---\r\n");
            report_zigbee_network_state_to_mcu(NWK_NOT_JOIN, FALSE);
            break;
        }
        case TAL_ZG_NWK_MF_TEST_LEAVE: {
            tal_main_debug("---nwk_mf_test_leave---\r\n");
            break;
        }
        default: {
            break;
        }
    }
}

VOID_T tal_zg_reset_factory_default_callback(TAL_RESET_TYPE_T type)
{
    tal_main_debug("receive reset to factory default cmd %d\r\n", type);
    if (TAL_RESET_FACTORY_AND_LEAVE == type) {
        #if COMMON_LINK_SCENE
            app_user_scene_info_clear(0xffff);
        #endif
        uart_mcu_reset_to_factory_send();
    }
}

VOID_T tal_beacon_mf_test_callback(VOID_T)
{
    tal_main_debug("enter beacon test\r\n");
    baudrate_adaptive_start(10, BAUDRATE_ADAPTIVE_TYPE_BEACON);
}

VOID_T tal_mf_test_pre_start_callback(VOID_T *args)
{
    tal_main_debug("enter dongle-beacon start\r\n");
}

VOID_T tal_mf_test_end_callback(UINT8_T *data)
{
    tal_main_debug("enter dongle-beacon end, value %d\r\n", *data);
}

/**
 * @brief recall scene callback
 *
 * @param[in]   ep_id:     endpoint id
 * @param[in]   scene_id:  scene id
 * @param[in]   group_id:  group id
 * @param[in]   time100ms: scene transition time(bat:100ms)
 * @param[in]   data:      point to scene data
 * @return  VOID_T
 */
VOID_T tal_zg_scene_recall_callback(UINT8_T ep_id, UINT8_T scene_id, UINT16_T group_id, UINT_T time100ms, TAL_SCENE_DATA_T *data)
{
    tal_log_print_raw("scene recall: ep %d, sce %d, gp 0x%02x, type %d\r\n", ep_id, scene_id, group_id, data->type);
    UINT8_T payload[64] = {0};
    UINT8_T payloadLen = 0;
    UINT16_T seqNum = app_seq_num_get();
    payloadLen = data->len;
    payload[0] = (UINT8_T)(seqNum >> 8);
    payloadLen++;
    payload[1] = (UINT8_T)(seqNum & 0xFF);
    payloadLen++;

    tal_system_memcpy(&payload[2], data->pdata, data->len);
    air_private_cluster_command_parse_handle(SERIAL_PROTOCOL_DATA_REQ, ZG_BROADCAST_MODE, payload, payloadLen);
}

#include "tkl_platform_types.h"

BOOL_T app_print_get_cfg(UINT8_T *print_type, UINT8_T *disable_irq, GPIO_PORT_PIN_T *gpio)
{
    gpio->port = PORT_C;
    gpio->pin = PIN_4;
    *print_type = 0;
    *disable_irq = 1;
#ifdef UART_PRINTF_MODE
    return 1;
#else
    return 0;
#endif
}