/*
* @Author: Leon
* @LastEditors: Leon
* @file name: uart_protocol_handle.c
* @Description: serial protocol used by TuYa ZigBee modules and MCU communicated by UART
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Date: 2019-03-13 17:20:41
* @LastEditTime: 2020-03-26 10:23:41
*/
#include "tal_memory.h"
#include "tal_data_send.h"

#include "uart_data_received_handle.h"
#include "air_data_received_handle.h"
#include "baudrate_adaptive.h"
#include "dp_data_backup.h"

#include "uart_data_send.h"

#define  MAX_READ_DP_NUMBER    10
#define  MAX_DP_SEND_DATA      60
#define  MAX_DP_U8_ITMES       36
#define  MAX_DP_U32_ITMES      24
#define  MAX_DP_RAW_ITMES      4

typedef struct 
{
UINT8_T     dp_id;
UINT8_T     dp_type;
UINT8_T     dp_value;
}dp_u8_t;

typedef struct 
{
UINT8_T     dp_id;
UINT8_T     dp_type;
UINT32_T    dp_value;
}dp_u32_t;

typedef struct 
{
UINT8_T     dp_id;
UINT8_T     dp_type;
UINT8_T     dp_len;
UINT8_T     dp_data[60];
}dp_raw_t;

STATIC UINT8_T dp_u8_current_number  = 0;
STATIC UINT8_T dp_u32_current_number = 0;
STATIC UINT8_T dp_raw_current_number = 0;
STATIC BOOL_T  dp_u8_full = FALSE;
STATIC BOOL_T  dp_u32_full = FALSE;
STATIC BOOL_T  dp_raw_full = FALSE;

STATIC UINT8_T dp_u8_current_location  = 0;
STATIC UINT8_T dp_u32_current_location = 0;
STATIC UINT8_T dp_raw_current_location = 0;

STATIC dp_u8_t  dp_u8_array[MAX_DP_U8_ITMES];
STATIC dp_u32_t dp_u32_array[MAX_DP_U32_ITMES];
STATIC dp_raw_t dp_raw_array[MAX_DP_RAW_ITMES];

STATIC UINT8_T insert_dp_u8_data(dp_data_t *dp_data);
STATIC UINT8_T insert_dp_u32_data(dp_data_t *dp_data);
STATIC UINT8_T insert_dp_raw_data(dp_data_t *dp_data);
STATIC BOOL_T insert_dp_data_to_array(dp_data_t *dp_data);

STATIC VOID_T send_dp_read_data(UINT8_T *data, UINT8_T data_len);

/**
 * @description: my atoi
 * @param[in] s[]: sorce char
 * @return: none
 */
STATIC UINT8_T insert_dp_u8_data(dp_data_t *dp_data)
{
  UINT8_T i = 0;
  for(i=0;i<dp_u8_current_number; i++){
    if(dp_u8_array[i].dp_id == dp_data->dpid){
      return i;
    }
  }
  
  if(dp_u8_current_number < MAX_DP_U8_ITMES){
    i= dp_u8_current_number;
    dp_u8_current_number++;
  }
  else{
    dp_u8_full = TRUE;
    i= dp_u8_current_location;
    dp_u8_current_location++;
    dp_u8_current_location = dp_u8_current_location % MAX_DP_U8_ITMES;
  }
  return i;
}

STATIC UINT8_T insert_dp_u32_data(dp_data_t *dp_data)
{
  UINT8_T i = 0;
  
  for(i=0;i<dp_u32_current_number; i++){
    if(dp_u32_array[i].dp_id == dp_data->dpid){
      return i;
    }
  }
  
  if(dp_u32_current_number < MAX_DP_U32_ITMES){
    i= dp_u32_current_number;
    dp_u32_current_number++;
  }
  else{
    dp_u32_full = TRUE;
    i= dp_u32_current_location;
    dp_u32_current_location++;
    dp_u32_current_location = dp_u32_current_location%MAX_DP_U8_ITMES;
  }
  return i;
}

STATIC UINT8_T insert_dp_raw_data(dp_data_t *dp_data)
{
  UINT8_T i = 0;
  
  for(i=0;i<dp_raw_current_number; i++){
    if(dp_raw_array[i].dp_id == dp_data->dpid){
      return i;
    }
  }
  
  if(dp_raw_current_number < MAX_DP_RAW_ITMES){
    i= dp_raw_current_number;
    dp_raw_current_number++;
  }
  else{
    dp_raw_full = TRUE;
    i= dp_raw_current_location;
    dp_raw_current_location++;
    dp_raw_current_location = dp_raw_current_location % MAX_DP_RAW_ITMES;
  }
  return i;
}

STATIC BOOL_T insert_dp_data_to_array(dp_data_t *dp_data)
{
  UINT8_T  itme = 0;
  switch(dp_data->dp_type){
  case DP_TYPE_BOOL:
  case DP_TYPE_ENUM:
    itme = insert_dp_u8_data(dp_data);
    dp_u8_array[itme].dp_id = dp_data->dpid;
    dp_u8_array[itme].dp_type = dp_data->dp_type;
    dp_u8_array[itme].dp_value = dp_data->dp_data[0];
    break;
  case DP_TYPE_VALUE:
    itme = insert_dp_u32_data(dp_data);
    dp_u32_array[itme].dp_id = dp_data->dpid;
    dp_u32_array[itme].dp_type = dp_data->dp_type;
    dp_u32_array[itme].dp_value = (((UINT32_T)dp_data->dp_data[0])<<24) 
      + (((UINT32_T)dp_data->dp_data[1])<<16)+(((UINT32_T)dp_data->dp_data[2])<<8)
        +(UINT32_T)dp_data->dp_data[3];
    break;
  case  DP_TYPE_RAW:
  case  DP_TYPE_STRING:
    itme = insert_dp_raw_data(dp_data);
    dp_raw_array[itme].dp_type = dp_data->dp_type;
    dp_raw_array[itme].dp_id = dp_data->dpid;
    dp_raw_array[itme].dp_len = (UINT8_T)dp_data->dp_len;
    tal_system_memcpy(dp_raw_array[itme].dp_data,dp_data->dp_data,dp_raw_array[itme].dp_len);
    break;
  default:
    break;
  }
  return TRUE;
}

/**
 * @description: dp data request handler
 * @param[in] buffer: dp data
 * @param[in] buffer_len: buff len
 * @return: none
 */
STATIC VOID_T send_dp_read_data(UINT8_T *data, UINT8_T data_len)
{
  TAL_ZG_SEND_DATA_T send_data;
  
  if((data == NULL)||(data_len > 62)){
    return;
  }
  
  tal_system_memset(&send_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
  
  send_data.zcl_id = (UINT8_T)app_seq_num_get();
  send_data.qos = QOS_0;
  send_data.addr.mode = SEND_MODE_DEV;
  send_data.addr.type.dev.cluster_id = CLUSTER_PRIVATE_TUYA_CLUSTER_ID;
  send_data.addr.type.dev.src_ep = 0X01;
  send_data.delay_time = 0;
  send_data.random_time = 0;
  send_data.frame_type = ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER;
  send_data.direction = ZG_ZCL_DATA_SERVER_TO_CLIENT;
  
  send_data.command_id = SERIAL_PROTOCOL_DATA_RSP;
  send_data.data.private.data[0] = 0;
  send_data.data.private.data[1] = send_data.zcl_id;
  
  tal_system_memcpy(&send_data.data.private.data[2], data, data_len);
  send_data.data.private.len = data_len+2;
  
  tal_zg_send_data(&send_data, NULL, 1000);
}

/**
 * @description: dp data request handler
 * @param[in] buffer: dp data
 * @param[in] buffer_len: buff len
 * @return: none
 */
VOID_T dev_data_req_handle(UINT8_T* buffer, UINT8_T buffer_len)
{
  UINT8_T i = 0;
  dp_data_t dp_inf;
  UINT8_T *pTr = NULL;
  
  if ((NULL == buffer) || (buffer_len < 2)){
    return;
  }
  
  pTr = buffer;
  
  for (i = 0; i < buffer_len;) {
    tal_system_memset(&dp_inf, 0, sizeof(dp_data_t));
    
    dp_inf.dpid = pTr[i];
    dp_inf.dp_type = (DP_TYPE_E)pTr[i + 1];
    dp_inf.dp_len = pTr[ i + 2] << 8;
    dp_inf.dp_len |= pTr[ i + 3];
    
    if ((dp_inf.dp_len < 60) && (dp_inf.dp_len > 0)) {
      tal_system_memcpy(dp_inf.dp_data, &pTr[i+4], dp_inf.dp_len);
    }
    else{
      return;
    }
    
    insert_dp_data_to_array(&dp_inf);
    
    i += (dp_inf.dp_len + 4);
    if (buffer_len  - i < 7){
      break;
    }
  }
}

/**
 * @description: dp data read handler
 * @param[in] buffer: dp data
 * @param[in] buffer_len: buff len
 * @return: none
 */
VOID_T read_dp_data_handle(UINT8_T *data, UINT8_T data_len)
{
  UINT8_T  i;
  UINT8_T  j;
  UINT8_T poll_times = 0;
  UINT8_T send_data_buffer[64]; 
  UINT8_T send_data_offset = 2;
  UINT8_T find_dp = 0;
  UINT8_T *send_raw_data;
  UINT8_T not_find_dp[12] = {0};
  UINT8_T not_find_dp_number = 0;
  
  for(i=0;i<data_len;i++){
    find_dp = 0;
    
    if(dp_u8_full == FALSE){
      poll_times = dp_u8_current_number;
    }
    else{
      poll_times = MAX_DP_U8_ITMES;
    }
    
    for(j=0;j<poll_times;j++){
      if(data[i] == dp_u8_array[j].dp_id){
        find_dp = 1;
        if(send_data_offset+5 > MAX_DP_SEND_DATA){
          send_dp_read_data(send_data_buffer,send_data_offset);
          send_data_offset = 0;
        }
        send_data_buffer[send_data_offset++] = dp_u8_array[j].dp_id;
        send_data_buffer[send_data_offset++] = dp_u8_array[j].dp_type;
        send_data_buffer[send_data_offset++] = 0;
        send_data_buffer[send_data_offset++] = 1;
        send_data_buffer[send_data_offset++] = dp_u8_array[j].dp_value;
        break;
      }
    }
    
    if(find_dp == 0){
      if(dp_u32_full == FALSE){
        poll_times = dp_u32_current_number;
      }
      else{
        poll_times = MAX_DP_U32_ITMES;
      }
      
      for(j=0;j<poll_times;j++){
        if(data[i] == dp_u32_array[j].dp_id){
          find_dp = 1;
          if(send_data_offset+8 > MAX_DP_SEND_DATA){
            send_dp_read_data(send_data_buffer,send_data_offset);
            send_data_offset = 0;
          }
          send_data_buffer[send_data_offset++] = dp_u32_array[j].dp_id;
          send_data_buffer[send_data_offset++] = dp_u32_array[j].dp_type;
          send_data_buffer[send_data_offset++] = 0;
          send_data_buffer[send_data_offset++] = 4;
          send_data_buffer[send_data_offset++] = dp_u32_array[j].dp_value >>24;
          send_data_buffer[send_data_offset++] = dp_u32_array[j].dp_value>>16;
          send_data_buffer[send_data_offset++] = dp_u32_array[j].dp_value>>8;
          send_data_buffer[send_data_offset++] = dp_u32_array[j].dp_value&0xff;
          break;
        }
      }
    }
    
    if(find_dp == 0){
      if(dp_raw_full == FALSE){
        poll_times = dp_raw_current_number;
      }
      else{
        poll_times = MAX_DP_RAW_ITMES;
      }
      
      for(j=0;j<poll_times;j++){
        if(data[i] == dp_raw_array[j].dp_id){
          find_dp = 1;
          send_raw_data = tal_malloc(sizeof(UINT8_T)*(dp_raw_array[j].dp_len+3));
          if(send_raw_data != NULL){
            send_raw_data[send_data_offset++] = dp_raw_array[j].dp_id;
            send_raw_data[send_data_offset++] = dp_raw_array[j].dp_type;
            send_raw_data[send_data_offset++] = 0;
            send_raw_data[send_data_offset++] = dp_raw_array[j].dp_len;
            tal_system_memcpy(&send_raw_data[send_data_offset],dp_raw_array[j].dp_data,dp_raw_array[j].dp_len);
            send_dp_read_data(send_data_buffer,send_data_offset);
            tal_free(send_raw_data);
          }
          break;
        }
      }
    }
    
    if(find_dp == 0){
      not_find_dp[not_find_dp_number++] = data[i];
    }
  }
  
  if(not_find_dp_number){
    uart_gateway_read_dp_value_send(not_find_dp,not_find_dp_number, FALSE);
  }
}

/**
 * @description: backup dp data init
 * @param[in] NULL
 * @return: none
 */
VOID_T backup_dp_data_init(VOID_T)
{
  tal_system_memset(&dp_u8_array,0,sizeof(dp_u8_array));
  tal_system_memset(&dp_u32_array,0,sizeof(dp_u32_array));
  tal_system_memset(&dp_raw_array,0,sizeof(dp_raw_array));
}

