/**
* @Author: zhangshu
* @email: shiliang.zhang@tuya.com
* @LastEditors: zhangshu
* @file name: uart_data_received_handle.c
* @Description: uart data receive handle
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-11-13
* @LastEditTime: 2022-11-13
*/
#include "tuya_jsmn.h"
#include "tkl_rf_data_transceiver.h"
#include "tkl_zcl_scene.h"

#include "tal_sw_timer.h"
#include "tal_memory.h"
#include "tal_data_send.h"
#include "tkl_network_manager.h"
#include "tal_network_mgr.h"
#include "tal_error_code.h"
#include "tal_attribute_rw.h"
#include "tal_uart.h"
#include "tal_system.h"

#include "app_common.h"
#include "app_user_flash.h"

#include "uart_data_send.h"
#include "uart_data_received_handle.h"

#include "air_data_send.h"
#include "air_data_received_handle.h"

#include "baudrate_adaptive.h"

#define UART_PROTOCOL_HANDLE_DBG 0

#if UART_PROTOCOL_HANDLE_DBG
#define uart_protocol_handle_debug(...)      tal_main_debug(__VA_ARGS__)
#else
#define uart_protocol_handle_debug(...)
#endif

#define DATA_CONF_MAX            32
#define SEQ_TIMEOUT_TICK         3000
#define ZIGBEE_JOIN_MAX_TIMEOUT            180*1000

typedef struct
{
    BOOL_T flag;
    UINT16_T zcl_seq_num;
    UINT16_T confirm_seq_num;
    UINT32_T currentTime;
}data_confirm_t;

STATIC data_confirm_t data_conf[DATA_CONF_MAX];
STATIC UINT8_T data_conf_items = 0;

//uart protocol handle 
TIMER_ID uart_rx_error_event_id;
TIMER_ID gw_net_st_check_timeout_event_id;
TIMER_ID power_on_report_mcu_version_event_id;
TIMER_ID reset_device_event_id;
TIMER_ID rf_test_timeout_event_id;
TIMER_ID user_rf_test_send_event_id;
TIMER_ID check_seq_queue_event_id;

STATIC VOID_T uart_received_error_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T gw_net_st_check_timeout_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T power_on_report_mcu_version_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T reset_device_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T rf_test_timeout_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T user_rf_test_send_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T check_seq_queue_event_handler(TIMER_ID timer_id, VOID_T *arg);

STATIC VOID_T protocol_recv_handle(UINT8_T data);
STATIC VOID_T uart_command_rx_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_data_receive_handler(UINT8_T *payload, UINT16_T payload_len);

//serial cmd handler
STATIC VOID_T uart_product_info_rsp_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_zigbee_cfg_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_ack_gateway_send_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_mcu_data_report_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_function_test_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_mcu_version_rsp_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_mcu_ota_cmd_handler(UINT8_T receive_cmd_id, uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_mcu_check_zigbee_network_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_time_get_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_check_gw_network_st_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_set_zigbee_network_info_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_mcu_data_broadcast_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_beacon_test_notify_cmd_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_set_from_wakeup_to_send_time_cmd_handler(uart_protocol_cmd_t *receive_cmd);

#if COMMON_LINK_SCENE
STATIC VOID_T tuya_exchange_value(UINT8_T * a, UINT8_T * b);
STATIC VOID_T local_scene_recall(UINT8_T endpoint, UINT16_T groupId, UINT8_T sceneId);
STATIC VOID_T uart_scene_switch_id_report_handler(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T uart_standard_multicast_cmd_handler(uart_protocol_cmd_t *receive_cmd);
#endif
STATIC VOID_T uart_device_config_callback(UINT8_T *config);

STATIC VOID_T dev_serial_rf_recv_callback(UINT8_T *packet, UINT8_T lqi, INT8_T rssi);
STATIC VOID_T received_set_zigbee_rf(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T app_ack_receive_callback(TAL_SEND_ST_E st, TAL_ZG_SEND_DATA_T *data);
STATIC VOID_T updata_seq_queue(UINT16_T zcl_seq_num, BOOL_T res);
STATIC BOOL_T add_seq_to_queue(UINT16_T confirm_seq_num, UINT16_T zcl_seq_num);
STATIC VOID_T uart_protocol_handle_event_init(VOID_T);

#ifndef TUYA_ZG_ROUTER
MODULE_CONFIG_INFO_T g_modle_network_info;
STATIC UINT16_T sg_wake_up_time = 0;
STATIC VOID_T receive_set_wake_up_time_info(uart_protocol_cmd_t *receive_cmd);
STATIC VOID_T received_set_zigbee_network_info(uart_protocol_cmd_t *receive_cmd);
#endif

extern VOID_T __mf_test_rf_init(UINT8_T channel, TKL_RF_RECV_CB recv_cb);

extern UINT8_T tal_zg_scene_match_index_get(UINT8_T ep_id, UINT16_T group_id, UINT8_T scene_id, TKL_SCENE_T *entry);
extern BOOL_T tkl_rf_transeiver_start(TKL_RF_RECV_CB recv_cb);
extern BOOL_T tkl_rf_channel_set(UINT8_T channel);
extern BOOL_T tkl_rf_data_send(UINT8_T *packet);
extern BOOL_T tkl_rf_transeiver_end(VOID_T);

//UART receivew
STATIC UINT8_T   g_uart_rcv_data[PROTOCOL_EXT_SIZE + MAX_SERIAL_PAYLOAD_SIZE];
STATIC UINT16_T  g_uart_rcv_data_len = 0;
STATIC UART_ST_T g_uart_rcv_st = UART_ST_HEAD_1;

STATIC WORK_STATE_E deviceWorkState = NWK_STATE_ERROR;
STATIC NWK_STATE_E networkSate = NWK_INFO_ERROR;

STATIC UINT8_T g_mcu_version = 0;
STATIC UINT8_T g_mcu_otaing_flag = 0;
STATIC UINT16_T  g_get_time_seq;
STATIC UINT16_T  g_get_gw_network_st_seq = 0;

STATIC UINT16_T rf_test_seq;
STATIC UINT8_T total_send_num = 0; ///<total send number in  RF test
STATIC UINT8_T total_receive_num = 0; ///<total receive number in  RF test
STATIC UINT8_T packet_payload[] = {0x55, 0xaa, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39};
STATIC UINT8_T g_group_broadcast_payload_buff[20] = {0};

/**
 * @brief uart protocol handle event init
 *
 * @param[in] none
 * @return none
 */
STATIC VOID_T uart_protocol_handle_event_init(VOID_T)
{
    tal_sw_timer_create(uart_received_error_event_handler, NULL, &uart_rx_error_event_id);
    tal_sw_timer_create(gw_net_st_check_timeout_event_handler, NULL, &gw_net_st_check_timeout_event_id);
    tal_sw_timer_create(power_on_report_mcu_version_event_handler, NULL, &power_on_report_mcu_version_event_id);
    tal_sw_timer_create(reset_device_event_handler, NULL, &reset_device_event_id);
    tal_sw_timer_create(rf_test_timeout_event_handler, NULL, &rf_test_timeout_event_id);
    tal_sw_timer_create(user_rf_test_send_event_handler, NULL, &user_rf_test_send_event_id);
    tal_sw_timer_create(check_seq_queue_event_handler, NULL, &check_seq_queue_event_id);
}

STATIC VOID_T power_on_report_mcu_version_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
	if(g_mcu_version > 0){
        air_mcu_version_report_send();
        uart_protocol_handle_debug("air send report mcu version \r\n");
	}
	else{
		uart_protocol_handle_debug("mcu version not get\n");
		tal_sw_timer_start(power_on_report_mcu_version_event_id, 5000, TAL_TIMER_ONCE);
	}
}

STATIC VOID_T reset_device_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    tal_system_reset();
}

STATIC VOID_T rf_test_timeout_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    if (total_receive_num > total_send_num)
    {
        total_receive_num = total_send_num;
    }

    uart_function_test_result_send(total_receive_num);
    total_receive_num = 0;
}

STATIC VOID_T check_seq_queue_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    UINT8_T i = 0;
    UINT32_T  time = tal_system_get_millisecond();

    if (data_conf_items > 0){
        for (i = 0; i < DATA_CONF_MAX; i++){
            if (data_conf[i].flag == TRUE){
                time = time - data_conf[i].currentTime;
                if (time >= SEQ_TIMEOUT_TICK){
                    data_conf_items--;
                    data_conf[i].flag = FALSE;
                    uart_mcu_data_report_send(data_conf[i].confirm_seq_num, FALSE);
                    break;
                }
            }
        }
        tal_sw_timer_start(check_seq_queue_event_id, 500, TAL_TIMER_ONCE);
    }
}

STATIC VOID_T user_rf_test_send_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    STATIC UINT8_T send_num = 0;
    UINT8_T test_data[32];

    send_num++;
    tal_system_memset(&test_data[0], 0, 32);
    test_data[0] = sizeof(packet_payload)+3;//3:length(1)+crc(2)
    tal_system_memcpy(&test_data[1], packet_payload, sizeof(packet_payload));
    if (total_send_num >= send_num) {
        tkl_rf_data_send(&test_data[0]);
        tal_sw_timer_start(user_rf_test_send_event_id, SCAN_RSSI_PERIOD, TAL_TIMER_ONCE);
    } else {
        send_num = 0;
        tkl_rf_transeiver_end();
        tal_sw_timer_stop(user_rf_test_send_event_id);
        tal_sw_timer_start(rf_test_timeout_event_id, 1, TAL_TIMER_ONCE);
    }
}

STATIC VOID_T uart_received_error_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    tal_error_code_set((TAL_ERR_CODE_E)UART_FRAME_ERROE_E_CODE);
    g_uart_rcv_st = UART_ST_HEAD_1;
}

/**
* @description: serial protocol send data confirm queue add
* @param {confirm_seq_num} the confirm sequence number form gateway
* @param {zcl_seq_num} the zcl sequence number of send data
* @return: true or false
*/
STATIC BOOL_T add_seq_to_queue(UINT16_T confirm_seq_num, UINT16_T zcl_seq_num)
{
    UINT8_T i = 0;
    uart_protocol_handle_debug("add seq to queue, confirm seq %x, zcl seq %x\r\n", confirm_seq_num, zcl_seq_num);
    for (i = 0; i < DATA_CONF_MAX; i++)
    {
        if (data_conf[i].flag == FALSE)
        {
            data_conf_items++;
            data_conf[i].flag = TRUE;
            data_conf[i].zcl_seq_num = zcl_seq_num;
            data_conf[i].confirm_seq_num = confirm_seq_num;
            data_conf[i].currentTime = tal_system_get_millisecond();
            if (!tal_sw_timer_is_running(check_seq_queue_event_id))
            {
                tal_sw_timer_start(check_seq_queue_event_id, 3000, TAL_TIMER_ONCE);
            }
            return  TRUE;
        }
    }
    return FALSE;
}

/**
* @description: serial protocol data queue confirm find function
* @param {zcl_seq_num} sequence number of the item need to find
* @return: the sequence number of find result item in queue
*/
STATIC VOID_T updata_seq_queue(UINT16_T zcl_seq_num, BOOL_T res)
{
    UINT8_T i = 0;
    uart_protocol_handle_debug("zcl seq %x, res %d\r\n", zcl_seq_num, res);
    for (i = 0; i < DATA_CONF_MAX; i++){
        if (data_conf[i].flag == TRUE){
            if (data_conf[i].zcl_seq_num == zcl_seq_num){
                data_conf[i].flag = FALSE;
                data_conf_items--;
                uart_protocol_handle_debug("foud and send confirm seq %x, res %d\r\n", data_conf[i].confirm_seq_num, res);
                uart_mcu_data_report_send((data_conf[i].confirm_seq_num), res);
                return;
            }
        }
    }
}


STATIC VOID_T protocol_recv_handle(UINT8_T data)
{
    STATIC UINT16_T temp_len;

    switch (g_uart_rcv_st){
        case UART_ST_HEAD_1:{
            if (UART_HEAD_1 == data){
                g_uart_rcv_data_len = 0;
                g_uart_rcv_data[g_uart_rcv_data_len++] = data;
                g_uart_rcv_st = UART_ST_HEAD_2;
            }
            break;
        }
        case UART_ST_HEAD_2:{
            if (UART_HEAD_2 == data){
                g_uart_rcv_data[g_uart_rcv_data_len++] = data;
                g_uart_rcv_st = UART_ST_VER;
            }
            else{
                g_uart_rcv_st = UART_ST_HEAD_1;
            }
            break;
        }
        case UART_ST_VER:{
            if (SERIAL_PROTOCOL_VER == data){
                g_uart_rcv_data[g_uart_rcv_data_len++] = data;
                g_uart_rcv_st = UART_ST_SEQ1;
            }
            else{
                g_uart_rcv_st = UART_ST_HEAD_1;
            }
        }
        break;
        case UART_ST_SEQ1:{
            g_uart_rcv_data[g_uart_rcv_data_len++] = data;
            g_uart_rcv_st = UART_ST_SEQ2;
        }
        break;
        case UART_ST_SEQ2:{
            g_uart_rcv_data[g_uart_rcv_data_len++] = data;
            g_uart_rcv_st = UART_ST_CMD;
        }
        break;
        case UART_ST_CMD:{
            g_uart_rcv_data[g_uart_rcv_data_len++] = data;
            g_uart_rcv_st = UART_ST_DATA_LEN_1;
        }
        break;
        case UART_ST_DATA_LEN_1:{
            g_uart_rcv_data[g_uart_rcv_data_len++] = data;
            g_uart_rcv_st = UART_ST_DATA_LEN_2;
            temp_len = ((UINT16_T)data) << 8;
            break;
        }
        case UART_ST_DATA_LEN_2:{
            g_uart_rcv_data[g_uart_rcv_data_len++] = data;
            temp_len += data;
            if (0 == temp_len){
                g_uart_rcv_st = UART_ST_CRC;
            }
            else if (temp_len <= MAX_SERIAL_PAYLOAD_SIZE){
                g_uart_rcv_st = UART_ST_DATA;
            }
            else{
                g_uart_rcv_st = UART_ST_HEAD_1;
            }
            break;
        }
        case UART_ST_DATA:{
            temp_len--;
            g_uart_rcv_data[g_uart_rcv_data_len++] = data;
            if (temp_len == 0){
                g_uart_rcv_st = UART_ST_CRC;
            }
            break;
        }
        case UART_ST_CRC:{
            g_uart_rcv_st = UART_ST_HEAD_1;
            UINT8_T sum = make_checksum(g_uart_rcv_data, g_uart_rcv_data_len);
            if (sum == data){
                uart_data_receive_handler(g_uart_rcv_data, g_uart_rcv_data_len);
            }
            tal_sw_timer_stop(uart_rx_error_event_id);
            break;
        }

        default:{
            g_uart_rcv_st = UART_ST_HEAD_1;
            break;
        }
    }

    if (g_uart_rcv_st != UART_ST_HEAD_1){
        tal_sw_timer_start(uart_rx_error_event_id, 100, TAL_TIMER_ONCE);
    }
}

STATIC VOID_T uart_data_receive_handler(UINT8_T *payload, UINT16_T payload_len)
{
    uart_protocol_cmd_t serial_protocol_receive_cmd;

    tal_system_memset(&serial_protocol_receive_cmd, 0, sizeof(uart_protocol_cmd_t));

    serial_protocol_receive_cmd.cmd_hdr = payload[HEAD_FIRST];
    serial_protocol_receive_cmd.cmd_hdr = serial_protocol_receive_cmd.cmd_hdr << 8 | payload[HEAD_SECOND];
    serial_protocol_receive_cmd.version = payload[PROTOCOL_VERSION];

    serial_protocol_receive_cmd.seq_num = payload[SEQ_LOCATION_1] << 8;
    serial_protocol_receive_cmd.seq_num |= payload[SEQ_LOCATION_2];

    serial_protocol_receive_cmd.cmd = payload[CMD_LOCATION];

    serial_protocol_receive_cmd.data_len = payload[LENGTH_HIGH] << 8;
    serial_protocol_receive_cmd.data_len += payload[LENGTH_LOW];

    if (serial_protocol_receive_cmd.data_len > MAX_SERIAL_PAYLOAD_SIZE){
        return;
    }
    tal_system_memcpy(serial_protocol_receive_cmd.data, &payload[DATA_START], serial_protocol_receive_cmd.data_len);

    /*serial protocol command handler*/
    uart_command_rx_handler(&serial_protocol_receive_cmd);
    return;
}

STATIC VOID_T gw_net_st_check_timeout_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    UINT8_T buffer[3] = {0, 0, 2}; // 2 表示超时
    UINT8_T bufferLen = 3;
    uart_gateway_network_st_req_cmd_response_send(buffer, bufferLen);
}

STATIC VOID_T uart_product_info_rsp_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    UINT8_T i = 0;
    CHAR_T productPid[33 + 1] = {0};
    CHAR_T productVer[33 + 1] = {0};
    UINT8_T start_location = 0;
    UINT8_T end_location = 0;
    CHAR_T tokType_manufacturer_name[MAX_MANU_NAME_SIZE] = {0};
    CHAR_T data[64] = {0};
    INT_T productAbility = 0;
    OPERATE_RET res = OPRT_COM_ERROR;

    for (i = 0; i < receive_cmd->data_len; i++){
        if (receive_cmd->data[i] == '{'){
            start_location = i;
        }
        if (receive_cmd->data[i] == '}'){
            end_location = i + 1;
        }
    }

    tal_system_memcpy(data, &(receive_cmd->data[start_location]), (end_location - start_location));
    res = tuya_json_get_string_value((CONST CHAR_T *)data, "p", productPid, sizeof(productPid));
    if (res){
    	uart_protocol_handle_debug("parse pid failure\n");
        return;
    }
    uart_protocol_handle_debug("productPid %s\r\n", productPid);
    res = tuya_json_get_string_value((CONST CHAR_T *)data, "v", productVer, sizeof(productVer));
    if (res){
    	uart_protocol_handle_debug("parse version failure\n");
        return;
    }
    uart_protocol_handle_debug("productVer %s\r\n", productVer);

    res = tuya_json_get_int_value((CONST CHAR_T *)data, "g", &productAbility);

    if (OPRT_OK == res) {
        uart_protocol_handle_debug("product ability group control\r\n");
        if(productAbility & 0x00000001){ // 0bit distinguish between single control and group control capabilities
            group_ability_set(1); // set to group control capabilities
        }
    } else if (OPRT_CJSON_PARSE_ERR == res){
        uart_protocol_handle_debug("product ability default unicast\r\n");
        group_ability_set(0); //set to single control capabilities
    }

    tal_zg_read_attribute(0x01, 
                          CLUSTER_BASIC_CLUSTER_ID,
                          ATTR_MANUFACTURER_NAME_ATTRIBUTE_ID,
                          tokType_manufacturer_name,
                          MAX_MANU_NAME_SIZE - 1);
    tal_system_memcpy(&tokType_manufacturer_name[9], productPid, DEFAULT_PID_LEN);

    tal_zg_write_attribute(0x01, 
                           CLUSTER_BASIC_CLUSTER_ID,
                           ATTR_MANUFACTURER_NAME_ATTRIBUTE_ID,
                           tokType_manufacturer_name,
                           ATTR_CHAR_STRING_ATTRIBUTE_TYPE);

    g_mcu_version = (my_atoi(&productVer[0]) << 6) & 0xC0;
    g_mcu_version += (my_atoi(&productVer[2]) << 4) & 0x30;
    g_mcu_version += my_atoi(&productVer[4]) & 0x0F;

    device_work_state_set(NWK_STATE_OK);
    baudrate_adaptive_success_callback();

    uart_protocol_handle_debug("mcu version %x\r\n", g_mcu_version);
    tal_sw_timer_start(power_on_report_mcu_version_event_id, 2000, TAL_TIMER_ONCE);
}

STATIC VOID_T uart_zigbee_cfg_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    /*reset the zigbee device or start the zigbee device to join network*/
    if ((1 == receive_cmd->data_len)){
        if ((receive_cmd->data[0] != 0x00) && (receive_cmd->data[0] != 0x01)) {
            return;
        }
    }
    else{
        return;
    }
    uart_zigbee_cfg_rsp_send(receive_cmd->seq_num);
    uart_device_config_callback(receive_cmd->data);
}

STATIC VOID_T uart_ack_gateway_send_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    UINT8_T state = 0;
    if ((TAL_ZG_NWK_JOIN_OK != tal_zg_nwk_status_get()) && (TAL_ZG_NWK_POWER_ON_ONLINE != tal_zg_nwk_status_get())
    		&& (TAL_ZG_NWK_REJOIN_OK != tal_zg_nwk_status_get())){
        state = 0x00;
    }
    else{
        state = 0x01;
    }

    uart_ack_gateway_rsp_send(state, receive_cmd->seq_num);
    air_mcu_DP_response_cmd_send(receive_cmd->data,receive_cmd->data_len,receive_cmd->seq_num, FALSE);
}

STATIC VOID_T uart_mcu_data_report_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    UINT8_T data_buf[MAX_ZIGBEE_PAYLOAD_SIZE] = {0};
    UINT8_T data_len = 0;

    if ((0 == receive_cmd->data_len) || (MAX_ZIGBEE_PAYLOAD_SIZE  < (receive_cmd->data_len + 2))){
        return;
    }

    data_buf[0] = (receive_cmd->seq_num >> 8) & 0xff;
    data_buf[1] = receive_cmd->seq_num & 0xff;

    tal_system_memcpy(&data_buf[2], receive_cmd->data, receive_cmd->data_len);

    data_len = receive_cmd->data_len + 2;

    if (data_len > MAX_ZIGBEE_PAYLOAD_SIZE){
        return;
    }

    if ((TAL_ZG_NWK_JOIN_OK != tal_zg_nwk_status_get()) 
        && (TAL_ZG_NWK_POWER_ON_ONLINE != tal_zg_nwk_status_get())
        && (TAL_ZG_NWK_REJOIN_OK != tal_zg_nwk_status_get())){
        uart_protocol_handle_debug("not joined, send report result 0\r\n");
        uart_mcu_data_report_send(receive_cmd->seq_num, 0x00);
        return;
    }

    add_seq_to_queue(receive_cmd->seq_num, ZCL_ID_DP_REPORT);

    air_mcu_DP_report_cmd_send(data_buf, data_len, app_ack_receive_callback);
}

STATIC VOID_T uart_function_test_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    UINT8_T scan_channel = receive_cmd->data[0];
    if ((scan_channel < 11) || (scan_channel > 26)){
        scan_channel = 11;
    }
    TAL_ZG_NWK_STATUS_E net_state = tal_zg_nwk_status_get();
    
    if ((TAL_ZG_NWK_JOIN_OK == net_state) 
        || (TAL_ZG_NWK_REJOIN_OK == net_state) 
        || (TAL_ZG_NWK_POWER_ON_ONLINE == net_state)
        || (TAL_ZG_NWK_JOIN_START == net_state)
        || (TAL_ZG_NWK_LOST == net_state)){
        
        return;
    }

    uart_protocol_handle_debug("start rf test\r\n");
    //__mf_test_rf_init(scan_channel, dev_serial_rf_recv_callback);

    tkl_rf_transeiver_start(dev_serial_rf_recv_callback);
    tkl_rf_channel_set(scan_channel);

    total_send_num = SERIAL_RF_SEND_NUM;
    total_receive_num = 0;
    rf_test_seq = receive_cmd->seq_num;

    tal_sw_timer_start(user_rf_test_send_event_id, SCAN_RSSI_PERIOD, TAL_TIMER_ONCE);
    tal_sw_timer_start(rf_test_timeout_event_id, SCAN_RSSI_TIMEOUT, TAL_TIMER_ONCE);
}

STATIC VOID_T uart_mcu_version_rsp_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if (receive_cmd->data_len == 1){
        g_mcu_version = receive_cmd->data[0];
        tal_sw_timer_start(power_on_report_mcu_version_event_id, 1, TAL_TIMER_ONCE);
    }
}

STATIC VOID_T uart_mcu_ota_cmd_handler(UINT8_T receive_cmd_id, uart_protocol_cmd_t *receive_cmd)
{
    UINT8_T send_cmd_id = 0;

    UINT8_T data_buffer[MAX_SERIAL_PAYLOAD_SIZE];
    UINT8_T data_length = 0;

    if ((0 >= receive_cmd->data_len) || (MAX_SERIAL_PAYLOAD_SIZE < receive_cmd->data_len)){
        return;
    }

    if (MAX_SERIAL_PAYLOAD_SIZE > (receive_cmd->data_len + 2)){
        data_buffer[0] = (receive_cmd->seq_num >> 8) & 0xff;
        data_buffer[1] = receive_cmd->seq_num & 0xff;

        tal_system_memcpy(&data_buffer[2], receive_cmd->data, receive_cmd->data_len);
        data_length = receive_cmd->data_len + 2;
    }
    else{
        return;
    }

    if (MCU_OTA_DATA_REQ_CMD == receive_cmd_id){
        send_cmd_id = SERIAL_PROTOCOL_MCU_OTA_DATA_REQ;
    }
    else if (MCU_OTA_END_CMD == receive_cmd_id){
        send_cmd_id = SERIAL_PROTOCOL_MCU_OTA_END;
        uart_mcu_ota_end_rsp_send(receive_cmd->seq_num);
    }
    air_mcu_ota_data_request_send(send_cmd_id, data_buffer, data_length, FALSE);
}

STATIC VOID_T uart_mcu_check_zigbee_network_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    UINT8_T network_state = 0;
    UINT8_T send_state = 0;

    network_state = tal_zg_nwk_status_get();
    if ((TAL_ZG_NWK_JOIN_OK == network_state) ||
            (TAL_ZG_NWK_POWER_ON_ONLINE == network_state)
            || (TAL_ZG_NWK_REJOIN_OK == network_state)
            || (TAL_ZG_NWK_LOST == network_state)){
        send_state = 0x01;
    }
    else if (TAL_ZG_NWK_JOIN_START == network_state){
        send_state = 0x03;
    }
    else if (NWK_STATE_ERROR == device_work_state_get()){
        send_state = 0x02;
    }
    else if ((TAL_ZG_NWK_POWER_ON_LEAVE == network_state)
             || (TAL_ZG_NWK_LOCAL_LEAVE  == network_state)
             || (TAL_ZG_NWK_REMOTE_LEAVE == network_state)
             || (TAL_ZG_NWK_JOIN_TIMEOUT == network_state)){
        send_state = 0x00;
    }

    uart_mcu_check_network_rsp_send(send_state, receive_cmd->seq_num);
}

STATIC VOID_T uart_time_get_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if ((NULL == receive_cmd) || (receive_cmd->data_len != 0)){
        return;
    }
    get_time_seq_set( receive_cmd->seq_num);  //mcu send time request, gateway response seq is not equ with it, so we need save it
    air_time_request_cmd_send((UINT8_T *) & (receive_cmd->seq_num), 2);
}

/*1.0.9 new add mcu check network state*/
STATIC VOID_T uart_check_gw_network_st_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if ((NULL == receive_cmd) || (receive_cmd->data_len != 0)){
        return;
    }
    tal_sw_timer_start(gw_net_st_check_timeout_event_id, GET_GW_NETWORK_ST_TIMEOUT, TAL_TIMER_ONCE);
    gw_network_st_seq_set( receive_cmd->seq_num);  //mcu send time request, gateway response seq is not equ with it, so we need save it
    air_gateway_network_status_request_send((UINT8_T *) & (receive_cmd->seq_num), 2);
}

STATIC VOID_T uart_set_zigbee_network_info_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if ((NULL == receive_cmd) || (receive_cmd->data_len != 0)){
        return;
    }
#ifndef TUYA_ZG_ROUTER
    received_set_zigbee_network_info(receive_cmd);
#else
    received_set_zigbee_rf(receive_cmd);/*1.0.9 new add mcu change tx power*/ 
#endif  
}

STATIC VOID_T uart_mcu_data_broadcast_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if ((NULL == receive_cmd->data) || (receive_cmd->data_len < 2)){
        return ;
    }

    uart_mcu_data_broadcast_send_rsp_send(receive_cmd->seq_num);

    air_mcu_DP_response_cmd_send(receive_cmd->data, receive_cmd->data_len, receive_cmd->seq_num, TRUE);
}

STATIC VOID_T uart_set_from_wakeup_to_send_time_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if ((NULL == receive_cmd->data) || (receive_cmd->data_len < 2)){
        return ;
    }

}

STATIC VOID_T uart_beacon_test_notify_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if ((NULL == receive_cmd->data) || (receive_cmd->data_len < 2)){
        return ;
    }
    baudrate_adaptive_success_callback();
}

#if COMMON_LINK_SCENE
STATIC VOID_T tuya_exchange_value(UINT8_T * a, UINT8_T * b)
{
    if(a == NULL || b == NULL){
        return;
    }
    UINT8_T temp = 0;
    temp = *a;
    *a = *b;
    *b = temp;
}

STATIC VOID_T local_scene_recall(UINT8_T endpoint, UINT16_T groupId, UINT8_T sceneId)
{
    UINT8_T payload[66] = {0}; //SCENES_MAX_SCENE_DATA_LEN + 2
    UINT8_T payloadLen = 0;
    UINT16_T seqNum = app_seq_num_get();
    TKL_SCENE_T *zcl_scene;

    UINT8_T match_index = tal_zg_scene_match_index_get(1, groupId, sceneId, zcl_scene);

    if (0xFF == match_index) {
        return;
    }
    if(zcl_scene){
        payload[0] = (UINT8_T)((seqNum >> 8) & 0xff);
        payload[1] = (UINT8_T)(seqNum & 0xff);
        payloadLen = zcl_scene->data_len + 2;

        tal_system_memcpy(&payload[2], zcl_scene->data, zcl_scene->data_len);
        air_private_cluster_command_parse_handle(SERIAL_PROTOCOL_DATA_REQ, ZG_BROADCAST_MODE, payload, payloadLen);
	}
}

STATIC VOID_T uart_scene_switch_id_report_handler(uart_protocol_cmd_t *receive_cmd)
{
    UINT16_T seq = 0;
    UINT8_T switchId = 0;
    UINT8_T group_id = 0;
    UINT8_T scene_id = 0;
    UINT8_T res = 0;
    UINT8_T buff[7];
    UINT8_T buff_len = 0;

    if (NULL == receive_cmd->data){
        return ;
    }

    switchId = receive_cmd->data[0];
    uart_protocol_handle_debug("switch id %d\r\n", switchId);
    if((switchId <= MAX_USER_SCENE_ITEM) && (switchId > 0)){
        if(app_switch_id_valid_check_and_get(switchId, &group_id, &scene_id)){
            air_standard_recall_scene_send(group_id, scene_id);
            res =1;
        }else{
            res = 0;
        }

        seq = app_seq_num_get();
        buff[0] = (UINT8_T)((seq >> 8) & 0xff);
        buff[1] = (UINT8_T)(seq & 0xff);
        buff[2] = switchId;
        buff[3] = 0x04;
        buff[4] = 0;
        buff[5] = 1;
        buff[6] = 0;
        buff_len = 7;

        air_mcu_DP_report_cmd_send(&buff[0], buff_len, NULL);
    }
    uart_mcu_report_switch_id_rsponse_send(receive_cmd->seq_num, res);
    if(res){
        local_scene_recall(1, group_id, scene_id);
    }
}

STATIC VOID_T uart_standard_multicast_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if(receive_cmd->data == NULL){
        return;
    }

    if(receive_cmd->data_len < 5){
        uart_multicast_response_send(receive_cmd->seq_num, STANDARD_MULTICAST_CMD, 0);
        return;
    }

    UINT16_T gid = ((UINT16_T)receive_cmd->data[0] << 8) | receive_cmd->data[1];
    UINT16_T cluster_sub = ((UINT16_T)receive_cmd->data[2] << 8) | receive_cmd->data[3];
    UINT8_T cmd = receive_cmd->data[4];
    UINT8_T * standard_payload = &(receive_cmd->data[5]);
    UINT8_T standard_len = (UINT8_T)(receive_cmd->data_len - 5);
    bool_t is_standard = FALSE;
    if(cluster_sub == CLUSTER_ON_OFF_CLUSTER_ID){
        switch(cmd){
            case CMD_OFF_COMMAND_ID:
            case CMD_ON_COMMAND_ID:
            case CMD_TOGGLE_COMMAND_ID:
                if(standard_len == 0){
                    is_standard = TRUE;
                }
                break;
            default:
                break;
        }
    }
    else if(cluster_sub == CLUSTER_LEVEL_CONTROL_CLUSTER_ID){
        switch(cmd){
            case CMD_MOVE_TO_LEVEL_COMMAND_ID:{
                if((standard_len >= 3) && (standard_len <= 5)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[1], &g_group_broadcast_payload_buff[2]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_COMMAND_ID:{
                if((standard_len >= 2) && (standard_len <= 4)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STEP_COMMAND_ID:{
                if((standard_len >= 4) && (standard_len <= 6)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STOP_COMMAND_ID:{
                if(standard_len <= 2){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_TO_LEVEL_WITH_ON_OFF_COMMAND_ID:{
                if((standard_len >= 3) && (standard_len <= 5)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[1], &g_group_broadcast_payload_buff[2]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_WITH_ON_OFF_COMMAND_ID:{
                if((standard_len >= 2) && (standard_len <= 4)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STEP_WITH_ON_OFF_COMMAND_ID:{
                if((standard_len >= 4) && (standard_len <= 6)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STOP_WITH_ON_OFF_COMMAND_ID:{
                if(standard_len <= 2){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            default:break;
        }
    }
    else if(cluster_sub == CLUSTER_COLOR_CONTROL_CLUSTER_ID){
        switch(cmd){
            case CMD_MOVE_TO_HUE_COMMAND_ID:{
                if((standard_len >= 4) && (standard_len <= 6)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_HUE_COMMAND_ID:{
                if((standard_len >= 2) && (standard_len <= 4)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STEP_HUE_COMMAND_ID:{
                if((standard_len >= 3) && (standard_len <= 5)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_TO_SATURATION_COMMAND_ID:{
                if((standard_len >= 3) && (standard_len <= 5)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[1], &g_group_broadcast_payload_buff[2]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_SATURATION_COMMAND_ID:{
                if((standard_len >= 2) && (standard_len <= 4)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STEP_SATURATION_COMMAND_ID:{
                if((standard_len >= 3) && (standard_len <= 5)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_TO_HUE_AND_SATURATION_COMMAND_ID:{
                if((standard_len >= 4) && (standard_len <= 6)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_TO_COLOR_COMMAND_ID:{
                if((standard_len >= 4) && (standard_len <= 6)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_COLOR_COMMAND_ID:{
                if((standard_len >= 4) && (standard_len <= 6)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[0], &g_group_broadcast_payload_buff[1]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STEP_COLOR_COMMAND_ID:{
                if((standard_len >= 6) && (standard_len <= 8)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[0], &g_group_broadcast_payload_buff[1]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[4], &g_group_broadcast_payload_buff[5]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_TO_COLOR_TEMPERATURE_COMMAND_ID:{
                if((standard_len >= 4) && (standard_len <= 6)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[0], &g_group_broadcast_payload_buff[1]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[2], &g_group_broadcast_payload_buff[3]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STOP_MOVE_STEP_COMMAND_ID:{
                if((standard_len <= 2)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_MOVE_COLOR_TEMPERATURE_COMMAND_ID:{
                if((standard_len >= 7) && (standard_len <= 9)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[1], &g_group_broadcast_payload_buff[2]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[3], &g_group_broadcast_payload_buff[4]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[5], &g_group_broadcast_payload_buff[6]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_STEP_COLOR_TEMPERATURE_COMMAND_ID:{
                if((standard_len >= 9) && (standard_len <= 11)){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[1], &g_group_broadcast_payload_buff[2]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[3], &g_group_broadcast_payload_buff[4]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[5], &g_group_broadcast_payload_buff[6]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[7], &g_group_broadcast_payload_buff[8]);
                    is_standard = TRUE;
                }
            }break;
            default:break;
        }
    }
    else if(cluster_sub == CLUSTER_WINDOW_COVERING_CLUSTER_ID){
        switch(cmd){
            case CMD_WINDOW_COVERING_UP_OPEN_COMMAND_ID:{
                if(standard_len == 0){
                    is_standard = TRUE;
                }
            }break;
            case CMD_WINDOW_COVERING_DOWN_CLOSE_COMMAND_ID:{
                if(standard_len == 0){
                    is_standard = TRUE;
                }
            }break;
            case CMD_WINDOW_COVERING_STOP_COMMAND_ID:{
                if(standard_len == 0){
                    is_standard = TRUE;
                }
            }break;
            case CMD_WINDOW_COVERING_GO_TO_LIFT_VALUE_COMMAND_ID:{
                if(standard_len == 2){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[0], &g_group_broadcast_payload_buff[1]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_WINDOW_COVERING_GO_TO_LIFT_PERCENTAGE_COMMAND_ID:{
                if(standard_len == 1){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            case CMD_WINDOW_COVERING_GO_TO_TILT_VALUE_COMMAND_ID:{
                if(standard_len == 2){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[0], &g_group_broadcast_payload_buff[1]);
                    is_standard = TRUE;
                }
            }break;
            case CMD_WINDOW_COVERING_GO_TO_TILT_PERCENTAGE_COMMAND_ID:{
                if(standard_len == 1){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    is_standard = TRUE;
                }
            }break;
            default:break;
        }
    }
    else if(cluster_sub == CLUSTER_SCENES_CLUSTER_ID){
        switch(cmd){
            case CMD_RECALL_SCENE_COMMAND_ID:{
                if(standard_len == 3){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[0], &g_group_broadcast_payload_buff[1]);
                    is_standard = TRUE;
                }
                else if(standard_len == 5){
                    tal_system_memcpy(g_group_broadcast_payload_buff, standard_payload, standard_len);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[0], &g_group_broadcast_payload_buff[1]);
                    tuya_exchange_value(&g_group_broadcast_payload_buff[3], &g_group_broadcast_payload_buff[4]);
                    is_standard = TRUE; 
                }
            }break;
            default:break;
        }
    }
    if(is_standard){
        uart_protocol_handle_debug("group ID %x, cluster id %x, cmd %x\r\n", gid, cluster_sub, cmd);
        air_group_broadcast_standard_cmd_send(gid, cluster_sub, cmd, g_group_broadcast_payload_buff, standard_len);
        uart_multicast_response_send(receive_cmd->seq_num, STANDARD_MULTICAST_CMD, 1);
    }
    else{
        uart_multicast_response_send(receive_cmd->seq_num, STANDARD_MULTICAST_CMD, 0);
    }
}

STATIC VOID_T uart_private_multicast_cmd_handler(uart_protocol_cmd_t *receive_cmd)
{
    if(receive_cmd->data == NULL){
        return;
    }

    UINT16_T gid = ((UINT16_T)receive_cmd->data[0] << 8) | receive_cmd->data[1];
    UINT8_T * dp_payload = &(receive_cmd->data[2]);
    UINT8_T dp_len = receive_cmd->data_len - 2;
    UINT8_T temp_len = 0;

    for(UINT8_T i = 0; i < dp_len;){
        temp_len += dp_payload[i + 2] * 0x100;
        temp_len += dp_payload[i + 3];
        temp_len += 4;
        i += (dp_payload[i + 2] * 0x100 + dp_payload[i + 3] + 4);
    }

    if(temp_len != dp_len){
        uart_multicast_response_send(receive_cmd->seq_num, PRIVATE_MULTICAST_CMD, 0);
        return;
    }

    air_group_broadcast_private_cmd_send(gid, dp_payload, dp_len);
    uart_multicast_response_send(receive_cmd->seq_num, PRIVATE_MULTICAST_CMD, 1);
}
#endif

STATIC VOID_T uart_command_rx_handler(uart_protocol_cmd_t *receive_cmd)
{
    switch (receive_cmd->cmd){
        case MCU_RESET_TO_FACTORY_CMD: {
            //mcu reset to factory result, do nothing
        }
        break;

        case PRODUCT_INFO_CMD:{
            uart_protocol_handle_debug("serial receive product info cmd rsp from mcu\r\n");
            uart_product_info_rsp_cmd_handler(receive_cmd);
        }
        break;

        case ZIGBEE_STATE_CMD:{  //report nwk state response  do nothing
            uart_protocol_handle_debug("serial receive zigbee state cmd rsp from mcu, do nothing\r\n");
            //send network state to mcu get mcu response do noting
        }
        break;

        case ZIGBEE_CFG_CMD:{
            uart_protocol_handle_debug("serial receive zigbee cfg cmd\r\n");
            uart_zigbee_cfg_cmd_handler(receive_cmd);
        }
        break;

        case ZIGBEE_DATA_REQ_CMD:{
            uart_protocol_handle_debug("serial receive data req cmd rsp from mcu, do nothing\r\n");
            //send dp to mcu get mcu response do noting
        }
        break;

        case ACK_GATEWAY_SEND_CMD:{
            uart_protocol_handle_debug("serial receive ack gateway send cmd rsp from mcu\r\n");
            uart_ack_gateway_send_cmd_handler(receive_cmd);
        }
        break;

        case DATA_REPORT_CMD:{
            uart_protocol_handle_debug("serial receive data report cmd\r\n");
            uart_mcu_data_report_cmd_handler(receive_cmd);
        }
        break;

        case FUNC_TEST_CMD:{
            uart_protocol_handle_debug("serial receive fucntion test cmd\r\n");
            uart_function_test_cmd_handler(receive_cmd);
        }
        break;

        case MCU_OTA_VER_REQ_CMD:{
            uart_protocol_handle_debug("serial receive mcu ota version req cmd rsp\r\n");
            uart_mcu_version_rsp_cmd_handler(receive_cmd);
        }
        break;

        case MCU_OTA_NOTIFY_CMD:{
            uart_protocol_handle_debug("serial receive mcu ota notify cmd rsp, do nothing\r\n");
        }
        break;

        case MCU_OTA_DATA_REQ_CMD:{
            uart_protocol_handle_debug("serial receive mcu ota data req cmd\r\n");
            uart_mcu_ota_cmd_handler(MCU_OTA_DATA_REQ_CMD, receive_cmd);
        }
        break;

        case MCU_OTA_END_CMD:{
            uart_protocol_handle_debug("serial receive mcu ota end cmd\r\n");
            uart_mcu_ota_cmd_handler(MCU_OTA_END_CMD, receive_cmd);
        }
        break;

        case CHECK_ZIGBEE_NETWORK_CMD:{
            uart_protocol_handle_debug("serial receive check zigbee network cmd\r\n");
            uart_mcu_check_zigbee_network_cmd_handler(receive_cmd);
        }
        break;

        case TIME_GET_CMD:{
            uart_protocol_handle_debug("serial receive time get cmd\r\n");
            uart_time_get_cmd_handler(receive_cmd);
        }
        break;

        case CHECK_GW_NETWORKING_ST_CMD:{ /*1.0.9 new add 0x25 mcu check network state*/
            uart_protocol_handle_debug("serial receive gateway networking state cmd\r\n");
            uart_check_gw_network_st_cmd_handler(receive_cmd);
            break;
        }

        case SET_ZIGBEE_NETWORK_INFO_CMD:
            uart_protocol_handle_debug("serial receive set zigbee network info cmd\r\n");
            uart_set_zigbee_network_info_cmd_handler(receive_cmd);
        break;

        case MCU_DATA_BROADCAST_SEND_CMD:{
            uart_protocol_handle_debug("serial receive mcu data broadcast send cmd\r\n");
            uart_mcu_data_broadcast_cmd_handler(receive_cmd);
        }
        break;

        case BEACON_TEST_NOTIFY_CMD:{
            uart_protocol_handle_debug("serial receive beacon test notify cmd rsp from mcu\r\n");
            uart_beacon_test_notify_cmd_handler(receive_cmd);
        }
        break;

        case ZIGBEE_DATA_REQ_GROUP_CMD: {
            uart_protocol_handle_debug("serial receive data req group cmd rsp from mcu, do nothing\r\n");
            //data request group command do noting
        }
        break;

        case SET_FROM_WAKE_UP_TO_SEND_TIME_CMD:{ // 0x2b mcu to zigbee module, config duration time in ms between awaken mcu and send data
            uart_protocol_handle_debug("serial receive set from wakeup to send time cmd\r\n");
            uart_set_from_wakeup_to_send_time_cmd_handler(receive_cmd);
        }
        break;

#if COMMON_LINK_SCENE
        case SCENE_SWITCH_NUM_GET_CMD:{
            uart_protocol_handle_debug("serial receive scene switch num get cmd rsp from mcu, do nothing\r\n");
        }
        break;

        case SCENE_SWITCH_ID_REPORT_CMD:{
            uart_protocol_handle_debug("serial receive scene switch id report cmd\r\n");
            uart_scene_switch_id_report_handler(receive_cmd);
            //need to do
        }
        break;

        case MULTICAST_SET_CMD: {
            uart_protocol_handle_debug("serial receive multicast set response, do nothing\r\n");
        }
        break;

        case STANDARD_MULTICAST_CMD:{
            uart_protocol_handle_debug("serial receive standard multicast cmd from mcu\r\n");
            uart_standard_multicast_cmd_handler(receive_cmd);
        }
        break;

        case PRIVATE_MULTICAST_CMD: {
            uart_protocol_handle_debug("serial receive private multicast cmd from mcu\r\n");
            uart_private_multicast_cmd_handler(receive_cmd);
        }
        break;
#endif

        default:
            uart_protocol_handle_debug("serial receive unsupport cmd!!!\r\n");
            break;
    }
}

STATIC VOID_T uart_device_config_callback(UINT8_T *config)
{
    WORK_STATE_E work_sate;
    if (0x00 == config[0]){
        tal_sw_timer_start(reset_device_event_id, 1000, TAL_TIMER_ONCE);
    }
    else if (0x01 == config[0]){
        work_sate = device_work_state_get();
        if(work_sate == NWK_STATE_OK) {
            if (tal_zg_nwk_status_get() != TAL_ZG_NWK_JOIN_START){
                uart_protocol_handle_debug("zigbee net start join\n");
                tal_zg_join_start(ZIGBEE_JOIN_MAX_TIMEOUT);
            }
        }
        else{
        	uart_zigbee_nwk_state_send(NWK_INFO_ERROR);
        }
    }
}

/**
* @description: application data send ack receive callback
* @param {st} send state
* @param {data} send data
* @return: none
*/
STATIC VOID_T app_ack_receive_callback(TAL_SEND_ST_E st, TAL_ZG_SEND_DATA_T *data)
{
    uart_protocol_handle_debug("tal send status %d\r\n", st);
    if (st == SEND_ST_OK){
        updata_seq_queue(data->zcl_id, TRUE);
    }
    else{
        tal_error_code_set((TAL_ERR_CODE_E)GATEWAY_RSP_E_CODE);
        updata_seq_queue(data->zcl_id, FALSE);
    }
}

STATIC VOID_T dev_serial_rf_recv_callback(UINT8_T *packet, UINT8_T lqi, INT8_T rssi)
{
    UINT8_T payload_len=0;
    if (packet == NULL){
        uart_protocol_handle_debug("packet null\r\n");
        return;
    }

    payload_len = packet[0];

    if (payload_len < 3){
        uart_protocol_handle_debug("payload len error\r\n");
        return;
    }

    uart_protocol_handle_debug("receive rf data, packet num %d, total receive number %d\r\n", packet[0], total_receive_num);
    if(tal_system_memcmp(&packet_payload[0], &packet[1], payload_len - 3))
    {
        uart_protocol_handle_debug("payload compare error\r\n");
       return;
    }

    tal_sw_timer_start(user_rf_test_send_event_id, 1, TAL_TIMER_ONCE);
    total_receive_num++;
}

STATIC VOID_T received_set_zigbee_rf(uart_protocol_cmd_t *receive_cmd)
{
    UINT8_T rf_power = 0;
    if (14 == receive_cmd->data_len){
        if(receive_cmd->data[13] == 0xfe){
            rf_power = DEFAULT_RF_POWER;
        }
        else if ((receive_cmd->data[13] >= MIN_RF_POWER) && (receive_cmd->data[13] <= MAX_RF_POWER)){
            rf_power = receive_cmd->data[13];
        }
    }

    app_user_flash_rf_power_write(rf_power);
    uart_set_zigbee_nwk_info_rsp_send(receive_cmd->seq_num);

    tal_sw_timer_start(reset_device_event_id, 1000, TAL_TIMER_ONCE);
}

#ifndef TUYA_ZG_ROUTER
STATIC VOID_T receive_set_wake_up_time_info(uart_protocol_cmd_t *receive_cmd)
{
    if(receive_cmd == NULL){
        return;
    }
    UINT16_T time = (UINT16_T)receive_cmd->data[0] << 8 | receive_cmd->data[1];
    if(time == 0xfffe){
        wake_up_time_set(DEFAULT_WAKE_UP_TIME);
        BOOL_T ret = app_user_flash_wakeup_time_write(DEFAULT_WAKE_UP_TIME);
        uart_set_wake_up_time_rsp_send(receive_cmd->seq_num, ret);
        return;
    }
    else if(time < MIN_WAKE_UP_TIME || time > MAX_WAKE_UP_TIME){
        uart_set_wake_up_time_rsp_send(receive_cmd->seq_num, FALSE);
        return;
    }
    wake_up_time_set(time);
    BOOL_T ret = app_user_flash_wakeup_time_write(time);
    uart_set_wake_up_time_rsp_send(receive_cmd->seq_num, ret);
}

STATIC VOID_T received_set_zigbee_network_info(uart_protocol_cmd_t *receive_cmd)
{
    MODULE_CONFIG_INFO_T received_modle_network_info;

    tal_system_memset(&received_modle_network_info, 0, sizeof(MODULE_CONFIG_INFO_T));

    if (14 == receive_cmd->data_len){
        received_modle_network_info.heartbeat_period_s = (UINT16_T)(receive_cmd->data[0] << 8) + (UINT16_T)(receive_cmd->data[1]);
        received_modle_network_info.join_timeout_s = (UINT16_T)(receive_cmd->data[2] << 8) + (UINT16_T)(receive_cmd->data[3]);
        received_modle_network_info.rejoin_interval_s = (UINT16_T)(receive_cmd->data[4] << 8) + (UINT16_T)(
                    receive_cmd->data[5]);
        received_modle_network_info.poll_interval_ms = (UINT16_T)(receive_cmd->data[6] << 8) + (UINT16_T)(receive_cmd->data[7]);
        received_modle_network_info.poweron_poll_period_s = (UINT16_T)(receive_cmd->data[8] << 8) + (UINT16_T)(
                    receive_cmd->data[9]);
        received_modle_network_info.poll_fail_max_times = receive_cmd->data[10];
        received_modle_network_info.trigger_rejoin_when_send_data = receive_cmd->data[11];
        received_modle_network_info.rejoin_max_times = receive_cmd->data[12];
        received_modle_network_info.tx_power = receive_cmd->data[13];


        if (received_modle_network_info.heartbeat_period_s == 0xfffe){
            g_modle_network_info.heartbeat_period_s = DEFAULT_HEART_PERIOD;
        }
        else if ((received_modle_network_info.heartbeat_period_s <= MAX_HEART_PERIOD) &&
                 (received_modle_network_info.heartbeat_period_s >= MIN_HEART_PERIOD)){
        	g_modle_network_info.heartbeat_period_s = received_modle_network_info.heartbeat_period_s;
        }

        if (received_modle_network_info.join_timeout_s == 0xfffe){
        	g_modle_network_info.join_timeout_s = DEFAULT_JION_PERIOD;
        }
        else if ((received_modle_network_info.join_timeout_s >= MIN_JION_PERIOD) &&
                 (received_modle_network_info.join_timeout_s <= MAX_JION_PERIOD)){
        	g_modle_network_info.join_timeout_s = received_modle_network_info.join_timeout_s;
        }

        if (received_modle_network_info.rejoin_interval_s == 0xfffe){
        	g_modle_network_info.rejoin_interval_s = DEFAULT_REJION_PERIOD;
        }
        else if ((received_modle_network_info.rejoin_interval_s >= MIN_REJION_PERIOD) &&
                 (received_modle_network_info.rejoin_interval_s <= MAX_REJION_PERIOD)){
        	g_modle_network_info.rejoin_interval_s = received_modle_network_info.rejoin_interval_s;
        }

        if (received_modle_network_info.poll_interval_ms == 0xfffe){
        	g_modle_network_info.poll_interval_ms = DEFAULT_POLL_PERIOD;
        	g_modle_network_info.close_poll = FALSE;
        }
        else if ((received_modle_network_info.poll_interval_ms <= MAX_POLL_PERIOD) &&
                 (received_modle_network_info.poll_interval_ms >= MIN_POLL_PERIOD)){
        	g_modle_network_info.poll_interval_ms = received_modle_network_info.poll_interval_ms;
        	g_modle_network_info.close_poll = FALSE;
        }
        else if (received_modle_network_info.poll_interval_ms == 0){
        	g_modle_network_info.close_poll = TRUE;
        }

        if (received_modle_network_info.poweron_poll_period_s == 0xfffe){
        	g_modle_network_info.poweron_poll_period_s = DEFAULT_POWER_ON_POLL_PERIOD;
        }
        else if ((received_modle_network_info.poweron_poll_period_s >= MIN_POWER_ON_POLL_PERIOD) &&
                 (received_modle_network_info.poweron_poll_period_s <= MAX_POWER_ON_POLL_PERIOD)){
        	g_modle_network_info.poweron_poll_period_s = received_modle_network_info.poweron_poll_period_s;
        }

        if (received_modle_network_info.poll_fail_max_times == 0xfe){
        	g_modle_network_info.poll_fail_max_times = DEFAULT_POLL_FAIL_TIMES;
        }
        else if ((received_modle_network_info.poll_fail_max_times >= MIN_POLL_FAIL_TIMES) &&
                 (received_modle_network_info.poll_fail_max_times <= MAX_POLL_FAIL_TIMES)){
        	g_modle_network_info.poll_fail_max_times = received_modle_network_info.poll_fail_max_times;
        }

        if (received_modle_network_info.trigger_rejoin_when_send_data == 0xfe){
        	g_modle_network_info.trigger_rejoin_when_send_data = 1;
        }
        else if ((received_modle_network_info.trigger_rejoin_when_send_data == 1) ||
                 (received_modle_network_info.trigger_rejoin_when_send_data == 0)){
        	g_modle_network_info.trigger_rejoin_when_send_data = received_modle_network_info.trigger_rejoin_when_send_data;
        }

        if (received_modle_network_info.rejoin_max_times == 0xfe){
        	g_modle_network_info.rejoin_max_times = DEFAULT_REJION_TRY_TIMES;
        }
        else if ((received_modle_network_info.rejoin_max_times >= MIN_REJION_TRY_TIMES) &&
                 (received_modle_network_info.rejoin_max_times <= MAX_REJION_TRY_TIMES)){
        	g_modle_network_info.rejoin_max_times = received_modle_network_info.rejoin_max_times;
        }

        if (received_modle_network_info.tx_power == 0xfe){
        	g_modle_network_info.tx_power = DEFAULT_RF_POWER;
        }
        else if ((received_modle_network_info.tx_power >= MIN_RF_POWER) &&
                 (received_modle_network_info.tx_power <= MAX_RF_POWER)){
        	g_modle_network_info.tx_power = received_modle_network_info.tx_power;
        }

        g_modle_network_info.valid = CHECK_RES_VALID;

        app_user_flash_modle_network_info_write(g_modle_network_info);
    }

    uart_set_zigbee_nwk_info_rsp_send(receive_cmd->seq_num);

    tal_sw_timer_start(reset_device_event_id, 1000, TAL_TIMER_ONCE);
}
#endif

/**
* @description: network state get
* @param none
* @return: network state
*/
NWK_STATE_E user_network_state_get(VOID_T)
{
    return networkSate;
}

/**
* @description: get get time seq
* @param {type} none
* @return: seq get time seq
*/
UINT16_T get_time_seq_get(VOID_T)
{
    return g_get_time_seq;
}

/**
* @description: set get time seq
* @param {seq} get time seq
* @return: none
*/
VOID_T get_time_seq_set(UINT16_T seq)
{
    g_get_time_seq = seq;
}

/**
* @description: get network status seq
* @param {type} none
* @return: seq network status seq
*/
UINT16_T gw_network_st_seq_get(VOID_T)
{
    return g_get_gw_network_st_seq;
}

/**
* @description: set network status seq
* @param {seq} network status seq
* @return: none
*/
VOID_T gw_network_st_seq_set(UINT16_T seq)
{
    g_get_gw_network_st_seq = seq;
}

/**
* @description: get current device work state
* @param {type} none
* @return: WORK_STATE_E current work state
*/
WORK_STATE_E device_work_state_get(VOID_T)
{
    return deviceWorkState;
}

/**
* @description: set device work state
* @param {state} WORK_STATE_E state
* @return: none
*/
VOID_T device_work_state_set(WORK_STATE_E state)
{
    deviceWorkState = state;
}

/**
* @description: get mcu version
* @param {type} none
* @return: mcu version
*/
UINT8_T mcu_version_get(VOID_T)
{
    return g_mcu_version;
}

/**
* @description: get rf test sequence numbers
* @param {type} none
* @return: sequence number
*/
UINT16_T rf_test_seq_get(VOID_T)
{
    return rf_test_seq;
}

/**
* @description: stop gateway status check timeout event
* @param none
* @return: none
*/
VOID_T stop_gw_st_check_timeout_event(VOID_T)
{
    tal_sw_timer_stop(gw_net_st_check_timeout_event_id);
}

/**
* @description: send network state to the MCU when the zigbee state is changed;
* @param {nwk_status} network status enum type, 0x00: not join, 0x01: joined;
* @return: none
*/
VOID_T report_zigbee_network_state_to_mcu(NWK_STATE_E nwk_state, BOOL_T send)
{
    if ((networkSate != nwk_state) || (send)){
        networkSate = nwk_state;
        uart_zigbee_nwk_state_send(nwk_state);
    }
}

#ifndef TUYA_ZG_ROUTER
/**
* @description: get wake up time
* @param {type} none
* @return: wake up time
*/
UINT16_T wake_up_time_get(VOID_T)
{
    return sg_wake_up_time;
}

/**
* @description: set wake up time
* @param {time} wake up time
* @return: none
*/
VOID_T wake_up_time_set(UINT16_T time)
{
    sg_wake_up_time = time;
}
#endif

/**
* @description: delay report mcu version
* @param {delay_ms} delay time in ms
* @return: none
*/
VOID_T delay_report_mcu_version(UINT32_T delay_ms)
{
    tal_sw_timer_start(power_on_report_mcu_version_event_id, delay_ms, TAL_TIMER_ONCE);
}

/**
* @description: uart rx callback
* @param {port_id} uart port id
* @param {buff} rx buff
* @param {len} rx len
* @return: none
*/
VOID_T uart_rx_callback(TUYA_UART_NUM_E port_id, VOID_T *buff, UINT16_T len)
{
    UINT16_T  index = 0;
    UINT8_T *ptr = NULL;

    ptr = buff;
    do{
        protocol_recv_handle(*ptr++);
        index++;
    }
    while (index < len);
}

/**
* @description: serial protocol send data confirm queue init
* @param {type} none
* @return: none
*/
VOID_T uart_protocol_handle_init(VOID_T)
{
    data_conf_items = 0;
    tal_system_memset(&data_conf, 0, sizeof(data_confirm_t)*DATA_CONF_MAX);
    uart_protocol_handle_event_init();
}

