/*
 * @Author: Leon
 * @LastEditors: Leon
 * @file name: serial_protocol.c
 * @Description: serial protocol used by TuYa ZigBee modules and MCU communicated by UART
 * @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
 * @Date: 2019-03-13 17:20:41
 * @LastEditTime: 2020-03-26 10:23:41
 */
#include "tal_memory.h"
#include "tal_system.h"
#include "tal_network_mgr.h"
#include "tal_sw_timer.h"
#include "tal_uart.h"

#include "app_common.h"
#include "uart_data_send.h"
#include "uart_data_received_handle.h"
#include "air_data_received_handle.h"

#define UART_DATA_SEND_DBG 0

#if UART_DATA_SEND_DBG
#define uart_data_send_debug(...)      tal_main_debug(__VA_ARGS__)
#else
#define uart_data_send_debug(...)
#endif

#define uart_malloc	tal_malloc
#define uart_free	tal_free

#define MAX_UART_QUEUE_NUM          24
#define MAX_UART_RETRY_TIMES        3
#define	MAX_POP_DATA_SIZE           128

typedef struct  uart_queue
{
    struct uart_queue  *next;
    UINT8_T     resend_times;
    UINT8_T     payload_len;
    UINT16_T    seq;
    UINT8_T     *payload;
}uart_queue_t;

typedef struct  uart_queue_set
{
    uart_queue_t *head;
    uart_queue_t *tail;
    UINT8_T    queue_number;
}uart_queue_set_t;

typedef enum
{
    UART_QUEUE_SEND_INIT,
    UART_QUEUE_SEND_BEGIN,
    UART_QUEUE_SEND_END
}UART_QUEUE_SEND_E;

TIMER_ID uart_send_queue_tick_event_id;

STATIC VOID_T uart_send_queue_tick_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T uart_queue_event_init(VOID_T);

UINT8_T g_pop_data[MAX_POP_DATA_SIZE];
UINT8_T g_pop_dataLen;
uart_queue_set_t uart_queue_set;

UART_QUEUE_SEND_E g_send_mcu_data = UART_QUEUE_SEND_INIT;

STATIC BOOL_T uart_command_generate(uart_protocol_cmd_t *send_cmd, UINT8_T *send_buf, UINT8_T *send_buf_len)
{
    UINT8_T index = 0, cmd_crc = 0;

    send_buf[index++] = ((send_cmd->cmd_hdr) >> 8) & 0xff;
    send_buf[index++] = (send_cmd->cmd_hdr) & 0xff;
    send_cmd->version = SERIAL_PROTOCOL_VER;

    send_buf[index++] = send_cmd->version;

    send_buf[index++] = (send_cmd->seq_num >> 8) & 0xff;
    send_buf[index++] = (send_cmd->seq_num) & 0xff;
    send_buf[index++] = send_cmd->cmd;

    send_buf[index++] = ((send_cmd->data_len) >> 8) & 0xff;
    send_buf[index++] = (send_cmd->data_len) & 0xff;

    if ((send_cmd->data_len) > (MAX_SERIAL_PAYLOAD_SIZE - index)){
        return FALSE;
    }

    tal_system_memcpy(&send_buf[index], send_cmd->data, send_cmd->data_len);
    index += send_cmd->data_len;

    cmd_crc = make_checksum(send_buf, index);

    send_buf[index++] = cmd_crc;
    *send_buf_len = index;
    return TRUE;
}

STATIC VOID_T uart_send_queue_tick_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
#ifndef TUYA_ZG_ROUTER
      BOOL_T pop_res = FALSE;
      if(UART_QUEUE_SEND_INIT == g_send_mcu_data){
        pop_res = pop_uart_queue(g_pop_data, &g_pop_dataLen);
        if(TRUE == pop_res){
          g_send_mcu_data = UART_QUEUE_SEND_BEGIN;
          //gpio reinit
          //todo: write mcu status
          //gpio_raw_output_write_status(zigbee_wakeup_mcu_config[0].port, zigbee_wakeup_mcu_config[0].pin, zigbee_wakeup_mcu_config[0].drive_flag); //clean the wakeup pin
          UINT32_T wake_up_time = wake_up_time_get();
          tal_sw_timer_start(uart_send_queue_tick_event_id, wake_up_time, TAL_TIMER_ONCE);
        }
        else{
            //zigbee_wakeup_mcu_config[0].out = GPIO_DOUT_HIGH;
            //zigbee_wakeup_mcu_config[0].mode = GPIO_MODE_INPUT_PULL;
            //gpio_raw_init(zigbee_wakeup_mcu_config[0]);
            //uart_data_send_debug("set zigbee wakeup mcu io input high\n");
            g_send_mcu_data = UART_QUEUE_SEND_INIT;
        }
      }
      else if(UART_QUEUE_SEND_BEGIN == g_send_mcu_data){
        g_send_mcu_data = UART_QUEUE_SEND_END;
        tal_uart_write(SERIAL_PROTOCOL_UART_ID, g_pop_data, g_pop_dataLen);
        tal_sw_timer_start(uart_send_queue_tick_event_id, 50, TAL_TIMER_ONCE);
       // uart_data_send_debug("UART_QUEUE_SEND_BEGIN\n");
      }
      else if(UART_QUEUE_SEND_END == g_send_mcu_data){
        g_send_mcu_data = UART_QUEUE_SEND_INIT;
        tal_sw_timer_start(uart_send_queue_tick_event_id, 25, TAL_TIMER_ONCE);
        //zigbee_wakeup_mcu_config[0].out = GPIO_DOUT_HIGH;
        //gpio_raw_init(zigbee_wakeup_mcu_config[0]);
        uart_data_send_debug("set zigbee wakeup mcu io output high\n");
      }
#else
      if(pop_uart_queue(g_pop_data, &g_pop_dataLen)){
        tal_uart_write(SERIAL_PROTOCOL_UART_ID, g_pop_data, g_pop_dataLen);
        tal_sw_timer_start(uart_send_queue_tick_event_id, 10, TAL_TIMER_ONCE);
      }
#endif
}

/**
 * @brief uart queue event init
 *
 * @param[in] none
 * @return none
 */
STATIC VOID_T uart_queue_event_init(VOID_T)
{
    tal_sw_timer_create(uart_send_queue_tick_event_handler, NULL, &uart_send_queue_tick_event_id);
}

BOOL_T push_uart_data(UINT8_T *data, UINT8_T len, UINT16_T seq, BOOL_T is_resend)
{
   UINT8_T *pbuffer = NULL;
   uart_queue_t *uart_queue_tmp = NULL;
  if(data == NULL){
    return FALSE;
  }
  if(len < 1){
    return FALSE;
  }

  if(uart_queue_set.queue_number < MAX_UART_QUEUE_NUM){
    pbuffer = (UINT8_T*)uart_malloc(sizeof(UINT8_T)*len);
    if(pbuffer != NULL){
    	uart_data_send_debug("queue pbuffer malloc\n");
      uart_queue_tmp = (uart_queue_t*)uart_malloc(sizeof(uart_queue_t));
      if(uart_queue_tmp != NULL){
        uart_queue_tmp->next = NULL;
        uart_queue_tmp->payload_len = len;
        uart_queue_tmp->seq = seq;
        uart_queue_tmp->payload = pbuffer;
        if(is_resend){
          uart_queue_tmp->resend_times = MAX_UART_RETRY_TIMES;
        }
        else{
          uart_queue_tmp->resend_times = 1;
        }
        tal_system_memcpy(pbuffer,data,len);

        if(uart_queue_set.queue_number == 0){
          uart_queue_set.tail = uart_queue_tmp;
          uart_queue_set.head = uart_queue_set.tail;
          uart_queue_set.tail->next = NULL;
          uart_queue_set.queue_number++;
        }
        else{
          uart_queue_set.tail->next = uart_queue_tmp;
          uart_queue_set.tail = uart_queue_tmp;
          uart_queue_set.queue_number++;
        }
        if(!tal_sw_timer_is_running(uart_send_queue_tick_event_id)){
            tal_sw_timer_start(uart_send_queue_tick_event_id, 5, TAL_TIMER_ONCE);
        }
      }
      else{
        uart_free(pbuffer);
      }
    }
  }
  return TRUE;
}

BOOL_T pop_uart_queue(UINT8_T *data, UINT8_T *data_len)
{
  uart_queue_t *tmp_free_head = NULL;

  if(uart_queue_set.queue_number){
    if(uart_queue_set.head != NULL){
      *data_len = (uart_queue_set.head->payload_len);
      tal_system_memcpy(data,(uart_queue_set.head->payload),*data_len);
      if(uart_queue_set.head->resend_times > 0){
        uart_queue_set.head->resend_times--;
      }
      if(uart_queue_set.head->resend_times == 0){
        tmp_free_head = uart_queue_set.head;
        uart_queue_set.head = uart_queue_set.head->next;
        uart_queue_set.queue_number--;
        uart_free(tmp_free_head->payload);
        uart_free(tmp_free_head);
      }
      return TRUE;
    }
    else{
      return FALSE;
    }
  }
  else{
    uart_queue_set.tail = NULL;
    if(uart_queue_set.head != NULL){
      uart_free(uart_queue_set.head->payload);
      uart_free(uart_queue_set.head);
    }
  }
  return FALSE;
}

VOID_T pop_all_data(VOID_T)
{
  UINT8_T buf[10];
  UINT8_T buf_len;
  STATIC UINT8_T times = 0;
  while(uart_queue_set.queue_number){
    times++;
    pop_uart_queue(buf,&buf_len);
  }
}


/**
 * serial reset to factory command send
 * @param[in]   NULL
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_mcu_reset_to_factory_send(VOID_T)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, sizeof(send_buf));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = app_seq_num_get();
    send_cmd.cmd = MCU_RESET_TO_FACTORY_CMD;

    send_cmd.data[0] = 0x01; //reset to factory
    send_cmd.data_len++;

    uart_data_send_debug("serial mcu reset to factory cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial product info requst command send
 * @param[in]   NULL
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_product_info_req_cmd_send(VOID_T)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    if (NWK_STATE_OK == device_work_state_get()){
        return FALSE;
    }
    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, sizeof(send_buf));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = app_seq_num_get();
    send_cmd.cmd = PRODUCT_INFO_CMD;
    send_cmd.data_len = 0;

    uart_data_send_debug("serial product info req cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial ZIGBEE network state send
 * @param[in]{nwk_status} zigbee network state
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_zigbee_nwk_state_send(UINT8_T nwk_status)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, sizeof(send_buf));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = app_seq_num_get();
    send_cmd.cmd = ZIGBEE_STATE_CMD;

    send_cmd.data[0] = nwk_status;
    send_cmd.data_len++;

    uart_data_send_debug("serial zigbee nwk state send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial ZIGBEE config response send
 * @param[in]{seqnum} zigbee config seq
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_zigbee_cfg_rsp_send(UINT16_T seqnum)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, 16);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seqnum;
    send_cmd.cmd = ZIGBEE_CFG_CMD;
    send_cmd.data_len = 0;

    uart_data_send_debug("serial zigbee cfg response send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial ack gateway response send
 * @param[in]{state} zigbee network state
 * @param[in]{seqnum} zigbee ack seq
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_ack_gateway_rsp_send(UINT8_T state, UINT16_T seq_num)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, 16);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seq_num;

    send_cmd.cmd = ACK_GATEWAY_SEND_CMD;

    send_cmd.data_len = 1;
    send_cmd.data[0] = state;

    uart_data_send_debug("serial ack gateway response send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial mcu data report send
 * @param[in]{res} response
 * @param[in]{seqnum} zigbee ack seq
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_mcu_data_report_send(UINT16_T seq, BOOL_T res)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[32];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, sizeof(send_buf));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seq;
    send_cmd.cmd = DATA_REPORT_CMD;
    send_cmd.data_len = 1;
    send_cmd.data[0] = res;

    uart_data_send_debug("serial mcu data report send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial function test result send
 * @param[in]{percentageRssi} zigbee rssi
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_function_test_result_send(UINT8_T percentageRssi)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[MAX_SERIAL_PAYLOAD_SIZE];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, MAX_SERIAL_PAYLOAD_SIZE);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = rf_test_seq_get();
    send_cmd.cmd = FUNC_TEST_CMD;

    send_cmd.data_len = 2;

    if (0 >= percentageRssi){
        send_cmd.data[0] = FALSE;
        send_cmd.data[1] = 0;
    }else{
        send_cmd.data[0] = TRUE;
        send_cmd.data[1] = percentageRssi;
    }

    uart_data_send_debug("serial function test result cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial gateway check mcu version send
 * @param[in]{NULL}
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_gateway_check_mcu_version_send(VOID_T)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[24];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, 24);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = app_seq_num_get();
    send_cmd.cmd = MCU_OTA_VER_REQ_CMD;
    send_cmd.data_len = 0;

    uart_data_send_debug("serial gateway check mcu version cmd send\r\n");
    if(uart_command_generate( &send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial gateway mcu ota notify cmd send
 * @param[in]{data} ota notify data
 * @param[in]{data_len} ota notify data length
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_gateway_mcu_ota_notify_cmd_send(UINT8_T* data, UINT8_T data_len)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[MAX_SERIAL_PAYLOAD_SIZE];
    UINT8_T send_buf_len = 0;

    if((NULL == data) ||(0 == data_len)||((MAX_SERIAL_PAYLOAD_SIZE-PROTOCOL_EXT_SIZE)<data_len))
    {
        return FALSE;
    }

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, MAX_SERIAL_PAYLOAD_SIZE);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = app_seq_num_get();
    send_cmd.cmd = MCU_OTA_NOTIFY_CMD;

    /*payload first and second byte is sequence number in version 2*/
    UINT16_T seqNum = 0;
    UINT8_T* ptr= NULL;

    ptr = data;/*the start three byte is frame control, sequence and command id*/
    seqNum = *ptr << 8 ;
    ptr++;
    seqNum |= *ptr;
    ptr++;

    send_cmd.seq_num = seqNum;
    send_cmd.data_len = data_len - 2;
    tal_system_memcpy(send_cmd.data, ptr, send_cmd.data_len);

    uart_data_send_debug("serial gateway mcu ota notify cmd send\r\n");
    if(uart_command_generate( &send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial gateway response mcu ota data request send
 * @param[in]{data} ota notify data
 * @param[in]{data_len} ota notify data length
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_gateway_response_mcu_ota_data_request_send(UINT8_T* data, UINT8_T data_len)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[MAX_SERIAL_PAYLOAD_SIZE];
    UINT8_T send_buf_len = 0;

    if((NULL == data) ||(0 == data_len)||((MAX_SERIAL_PAYLOAD_SIZE-PROTOCOL_EXT_SIZE)<data_len))
    {
        return FALSE;
    }

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, MAX_SERIAL_PAYLOAD_SIZE);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.cmd = MCU_OTA_DATA_REQ_CMD;

    /*payload first and second byte is sequence number in version 2*/
    UINT16_T seqNum = 0;
    UINT8_T* ptr= NULL;

    ptr = data;/*the start three byte is frame control, sequence and command id*/
    seqNum = *ptr << 8 ;
    ptr++;
    seqNum |= *ptr;
    ptr++;

    send_cmd.seq_num = seqNum;
    send_cmd.data_len = data_len - 2;
    tal_system_memcpy(send_cmd.data, ptr, send_cmd.data_len);

    uart_data_send_debug("serial gateway response mcu ota data cmd send\r\n");
    if(uart_command_generate( &send_cmd, send_buf, &send_buf_len)) {
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial mcu ota end response send
 * @param[in]{seqnum} mcu ota end seq
 * @param[in]{data_len} ota notify data length
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_mcu_ota_end_rsp_send(UINT16_T seqnum)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;
    
    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, 16);
    
    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seqnum;
    send_cmd.cmd = MCU_OTA_END_CMD;
    
    send_cmd.data[0] = 0x00;
    send_cmd.data_len++;

    uart_data_send_debug("serial mcu ota end response cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial mcu check network status response response send
 * @param[in]{state} mcu ota end seq
 * @param[in]{seqnum} mcu check network response seq
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_mcu_check_network_rsp_send(UINT8_T state, UINT16_T seqnum)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, sizeof(send_buf));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.cmd = CHECK_ZIGBEE_NETWORK_CMD;
    send_cmd.seq_num = seqnum;

    send_cmd.data[0] = state;
    send_cmd.data_len++;

    uart_data_send_debug("serial mcu check network response cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial get time response send
 * @param[in]{data} ota notify data
 * @param[in]{data_len} ota notify data length
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_time_get_cmd_response_send(UINT8_T* data, UINT8_T data_len)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[32];
    UINT8_T send_buf_len = 0;
    UINT8_T* ptr= NULL;

    if((NULL == data) || (data_len < 10)){
        return FALSE;
    }

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, 32);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = get_time_seq_get();
    send_cmd.cmd = TIME_GET_CMD;

    ptr = &data[2];/*the start three byte is frame control, sequence and command id*/
    send_cmd.data_len = (data_len - 2);
    tal_system_memcpy(send_cmd.data, ptr, send_cmd.data_len);

    uart_data_send_debug("serial time get response cmd send\r\n");
    if(uart_command_generate( &send_cmd, send_buf, &send_buf_len)){
    	push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }
    else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial gateway network status req command response send
 * @param[in]{data} ota notify data
 * @param[in]{data_len} ota notify data length
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_gateway_network_st_req_cmd_response_send(UINT8_T* data, UINT8_T data_len)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[32];
    UINT8_T send_buf_len = 0;
    UINT8_T* ptr= NULL;

    if((NULL == data) ||(0 == data_len)||((MAX_SERIAL_PAYLOAD_SIZE-PROTOCOL_EXT_SIZE)<data_len))
    {
        return FALSE;
    }
	
    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, 32);

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = gw_network_st_seq_get();
    send_cmd.cmd = CHECK_GW_NETWORKING_ST_CMD;

    ptr = &data[2];/*the start three byte is frame control, sequence and command id*/
    send_cmd.data_len = (data_len - 2);
    tal_system_memcpy(send_cmd.data, ptr, send_cmd.data_len);

    uart_data_send_debug("serial gateway network status req cmd send\r\n");
    if(uart_command_generate( &send_cmd, send_buf, &send_buf_len)){
    	push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial set zigbee network info response send
 * @param[in]{seqnum} set netowrk info seq
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_set_zigbee_nwk_info_rsp_send(UINT16_T seqnum)
{
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    uart_protocol_cmd_t send_cmd;
    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seqnum;
    send_cmd.cmd = SET_ZIGBEE_NETWORK_INFO_CMD;
    send_cmd.data[0] = 0x01;
    send_cmd.data_len++;

    uart_data_send_debug("serial set zigbee nwk info response cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial set zigbee network info response send
 * @param[in]{seqnum} set netowrk info seq
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_mcu_data_broadcast_send_rsp_send(UINT16_T seqnum)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;
    
    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, 16);
    
    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.cmd = MCU_DATA_BROADCAST_SEND_CMD;
    send_cmd.seq_num = seqnum;

    send_cmd.data[0] = 0x01;
    send_cmd.data_len++;

    uart_data_send_debug("serial mcu data broadcast cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial gateway read dp value send
 * @param[in]{data} ota notify data
 * @param[in]{data_len} ota notify data length
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_gateway_read_dp_value_send(UINT8_T* data, UINT8_T data_len, BOOL_T with_seq)
{
    uart_protocol_cmd_t send_cmd;
    UINT16_T seqNum = 0;
    UINT8_T* ptr= NULL;
    UINT8_T send_buf[64];
    UINT8_T send_buf_len = 0;

    if((NULL == data) ||(0 == data_len)||((MAX_SERIAL_PAYLOAD_SIZE-PROTOCOL_EXT_SIZE)<data_len))
    {
        return FALSE;
    }

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.cmd = READ_MCU_DP_DATA_CMD;

    if (with_seq) {
        ptr = data;/*the start three byte is frame control, sequence and command id*/
        seqNum = (UINT16_T)(*ptr) << 8;
        ptr++;
        seqNum |= *ptr;
        ptr++;

        send_cmd.seq_num = seqNum;
        ptr = &data[2];/*the start three byte is frame control, sequence and command id*/
        if(data_len>2){ // The first two bytes is seq really data begin from third byte
            send_cmd.data_len = (data_len - 2);
            tal_system_memcpy(send_cmd.data, ptr, send_cmd.data_len);
        }
    } else {
        send_cmd.seq_num = app_seq_num_get();
        send_cmd.data_len = data_len;
        tal_system_memcpy(send_cmd.data,data,data_len);
    }

    uart_data_send_debug("serial gateway read dp value cmd send\r\n");
    if(uart_command_generate( &send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }
    return TRUE;
}

/**
 * serial beacon notify command send
 * @param[in]   NULL
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_beacon_notify_send(VOID_T)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;
    //0x55, 0xaa, 0x02, 0x00, 0x01, 0x29, 0x00, 0x01, 0x00, 0x2c

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, sizeof(send_buf));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = app_seq_num_get();
    send_cmd.cmd = BEACON_TEST_NOTIFY_CMD;

    send_cmd.data[0] = 0x00; //beacon notify
    send_cmd.data_len++;

    uart_data_send_debug("serial beacon notify cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial gateway read dp value send
 * @param[in]{cmd} cmd from air zigbee cmd
 * @param[in]{data} ota notify data
 * @param[in]{data_len} ota notify data length
 * @param[in]{seqnum} air dp seq
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_air_zigbee_dp_req_send(UINT8_T cmd, UINT8_T *data, UINT8_T data_len, UINT16_T seqnum)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[MAX_SERIAL_PAYLOAD_SIZE] = {0};
    UINT8_T send_buf_len = 0;

    if((NULL == data) ||(0 == data_len)||((MAX_SERIAL_PAYLOAD_SIZE-PROTOCOL_EXT_SIZE)<data_len))
    {
        return FALSE;
    }

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));
    tal_system_memset(&send_buf, 0, sizeof(send_buf));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seqnum;
    send_cmd.cmd = cmd;

    send_cmd.data_len = data_len;
    tal_system_memcpy(send_cmd.data, data, send_cmd.data_len);

    uart_data_send_debug("serial air zigbee dp request cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

#ifndef TUYA_ZG_ROUTER
/**
 * serial gateway read dp value send
 * @param[in]{data} ota notify data
 * @param[in]{data_len} ota notify data length
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_set_wake_up_time_rsp_send(UINT16_T seqnum, BOOL_T response)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seqnum;
    send_cmd.cmd = SET_FROM_WAKE_UP_TO_SEND_TIME_CMD;

    send_cmd.data[0] = response;
    send_cmd.data_len++;

    uart_data_send_debug("serial set wake up time response cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}
#endif

#if COMMON_LINK_SCENE
/**
 * serial mcu report switch id response send
 * @param[in]{seqnum} seq number
 * @param[in]{response} response result
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_mcu_report_switch_id_rsponse_send(UINT16_T seqnum, BOOL_T response)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seqnum;
    send_cmd.cmd = SCENE_SWITCH_ID_REPORT_CMD;

    send_cmd.data[0] = response;
    send_cmd.data_len++;

    uart_data_send_debug("serial mcu report switch id response send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial mcu report switch id response send
 * @param[in]{kid} key id
 * @param[in]{gid} group id
 * @param[in]{sid} scene id
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_multicast_set_cmd_send(UINT8_T kid, UINT16_T gid, UINT8_T sid)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = app_seq_num_get();
    send_cmd.cmd = MULTICAST_SET_CMD;
    send_cmd.data[0] = kid;
    send_cmd.data[1] = (UINT8_T)(gid >> 8);
    send_cmd.data[2] = (UINT8_T)gid;
    send_cmd.data[3] = sid;
    send_cmd.data_len = 4;

    uart_data_send_debug("serial multicast set cmd send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}

/**
 * serial uart multicast response send
 * @param[in]{kid} key id
 * @param[in]{gid} group id
 * @param[in]{sid} scene id
 * @return      TRUE when success, else FALSE
 */
BOOL_T uart_multicast_response_send(UINT16_T seqnum, UINT8_T cmd, UINT8_T result)
{
    uart_protocol_cmd_t send_cmd;
    UINT8_T send_buf[16];
    UINT8_T send_buf_len = 0;

    tal_system_memset(&send_cmd, 0, sizeof(uart_protocol_cmd_t));

    send_cmd.cmd_hdr = SERIAL_PROTOCOL_HDR;
    send_cmd.version = SERIAL_PROTOCOL_VER;
    send_cmd.seq_num = seqnum;
    send_cmd.cmd = cmd;
    send_cmd.data[0] = result;
    send_cmd.data_len = 1;

    uart_data_send_debug("serial multicast response send\r\n");
    if (uart_command_generate(&send_cmd, send_buf, &send_buf_len)){
        push_uart_data(send_buf, send_buf_len, send_cmd.seq_num, FALSE);
    }else{
        return FALSE;
    }

    return TRUE;
}
#endif

/**
 * @brief uart queue init
 *
 * @param[in] none
 * @return none
 */
VOID_T uart_queue_init(VOID_T)
{
  tal_system_memset(&uart_queue_set, 0, sizeof(uart_queue_set_t));
  uart_queue_event_init();
}

